from django.test import TestCase

from tag.factories.tag import TagFactory
from tag.factories.tagged_item import TaggedProductFactory
from tag.managers import TaggedItemManager
from tag.models.tagged_item import TaggedItem
from product.factories.product import ProductFactory


class TestTaggedItemManager(TestCase):
    def setUp(self):
        self.active_tag = TagFactory(active=True)
        self.inactive_tag = TagFactory(active=False)
        self.product1 = ProductFactory()
        self.product2 = ProductFactory()

        self.active_tagged_item = TaggedProductFactory(
            tag=self.active_tag, content_object=self.product1
        )
        self.inactive_tagged_item = TaggedProductFactory(
            tag=self.inactive_tag, content_object=self.product2
        )

    def test_get_queryset_includes_optimizations(self):
        queryset = TaggedItem.objects.get_queryset()

        self.assertIn("tag", queryset.query.select_related)
        self.assertIn("content_type", queryset.query.select_related)

        self.assertIn("tag__translations", queryset._prefetch_related_lookups)

    def test_active_tags_method(self):
        active_items = TaggedItem.objects.active_tags()

        self.assertIn(self.active_tagged_item, active_items)

        self.assertNotIn(self.inactive_tagged_item, active_items)

        self.assertEqual(active_items.count(), 1)

    def test_get_tags_for_method(self):
        another_tag = TagFactory(active=True)
        another_tagged_item = TaggedProductFactory(
            tag=another_tag, content_object=self.product1
        )

        product1_items = TaggedItem.objects.get_tags_for(
            type(self.product1), self.product1.id
        )

        self.assertIn(self.active_tagged_item, product1_items)
        self.assertIn(another_tagged_item, product1_items)

        self.assertNotIn(self.inactive_tagged_item, product1_items)

        self.assertEqual(product1_items.count(), 2)

    def test_get_tags_for_different_content_types(self):
        items_for_product1 = TaggedItem.objects.get_tags_for(
            type(self.product1), self.product1.id
        )

        items_for_product2 = TaggedItem.objects.get_tags_for(
            type(self.product2), self.product2.id
        )

        self.assertEqual(items_for_product1.count(), 1)
        self.assertEqual(items_for_product2.count(), 1)

        self.assertNotEqual(list(items_for_product1), list(items_for_product2))

    def test_manager_returns_correct_type(self):
        self.assertIsInstance(TaggedItem.objects, TaggedItemManager)

        active_queryset = TaggedItem.objects.active_tags()
        self.assertTrue(hasattr(active_queryset, "filter"))
        self.assertTrue(hasattr(active_queryset, "exclude"))

    def test_chaining_methods(self):
        recent_active_items = (
            TaggedItem.objects.active_tags()
            .filter(created_at__isnull=False)
            .order_by("-created_at")
        )

        self.assertGreaterEqual(recent_active_items.count(), 0)

        for item in recent_active_items:
            self.assertTrue(item.tag.active)

    def test_performance_with_large_dataset(self):
        items = []
        for i in range(10):
            tag = TagFactory(active=(i % 2 == 0))
            product = ProductFactory()
            item = TaggedProductFactory(tag=tag, content_object=product)
            items.append(item)

        active_items = TaggedItem.objects.active_tags()
        active_count = active_items.count()

        self.assertGreaterEqual(active_count, 5)

        for item in active_items:
            self.assertTrue(item.tag.active)

    def test_empty_queryset_behavior(self):
        TaggedItem.objects.all().delete()

        active_items = TaggedItem.objects.active_tags()
        self.assertEqual(active_items.count(), 0)

        items_for_product = TaggedItem.objects.get_tags_for(
            type(self.product1), self.product1.id
        )
        self.assertEqual(items_for_product.count(), 0)

    def test_get_tags_for_nonexistent_object(self):
        items = TaggedItem.objects.get_tags_for(
            type(self.product1),
            99999,
        )

        self.assertEqual(items.count(), 0)

    def test_select_related_optimization(self):
        items = TaggedItem.objects.all()

        for item in items[:2]:
            tag_name = item.tag.active
            content_type_name = item.content_type.model

            self.assertIsNotNone(tag_name)
            self.assertIsNotNone(content_type_name)

    def test_prefetch_tag_translations(self):
        items = TaggedItem.objects.all()

        for item in items[:2]:
            tag_label = item.tag.safe_translation_getter(
                "label", any_language=True
            )
            self.assertTrue(isinstance(tag_label, (str, type(None))))


class TestTaggedItemManagerEdgeCases(TestCase):
    def test_manager_with_deleted_tags(self):
        tag = TagFactory(active=True)
        product = ProductFactory()
        TaggedProductFactory(tag=tag, content_object=product)

        tag_id = tag.id
        tag.delete()

        self.assertFalse(TaggedItem.objects.filter(tag_id=tag_id).exists())

    def test_manager_with_deleted_content_objects(self):
        tag = TagFactory(active=True)
        product = ProductFactory()
        TaggedProductFactory(tag=tag, content_object=product)

        product_id = product.id
        product.delete()

        self.assertTrue(
            TaggedItem.objects.filter(object_id=product_id).exists()
        )

    def test_get_tags_for_with_different_model_types(self):
        product_items = TaggedItem.objects.get_tags_for(
            ProductFactory._meta.model,
            self.product1.id if hasattr(self, "product1") else 1,
        )

        self.assertGreaterEqual(product_items.count(), 0)

    def test_manager_method_consistency(self):
        all_items = TaggedItem.objects.all()
        active_items = TaggedItem.objects.active_tags()

        self.assertLessEqual(active_items.count(), all_items.count())

        for item in active_items:
            self.assertTrue(item.tag.active)



from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from drf_spectacular.helpers import lazy_serializer
from drf_spectacular.utils import extend_schema_field
from parler_rest.serializers import TranslatableModelSerializer
from rest_framework import serializers
from rest_framework.relations import PrimaryKeyRelatedField

from blog.models.author import BlogAuthor
from blog.models.category import BlogCategory
from blog.models.post import BlogPost
from blog.models.tag import BlogTag
from blog.utils import calculate_reading_time
from core.api.schema import generate_schema_multi_lang
from core.utils.serializers import TranslatedFieldExtended

User = get_user_model()


@extend_schema_field(generate_schema_multi_lang(BlogPost))
class TranslatedFieldsFieldExtend(TranslatedFieldExtended):
    pass


class BlogPostSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[BlogPost]
):
    likes = PrimaryKeyRelatedField(queryset=User.objects.all(), many=True)
    category = PrimaryKeyRelatedField(queryset=BlogCategory.objects.all())
    tags = PrimaryKeyRelatedField(queryset=BlogTag.objects.all(), many=True)
    author = PrimaryKeyRelatedField(queryset=BlogAuthor.objects.all())
    translations = TranslatedFieldsFieldExtend(shared_model=BlogPost)

    reading_time = serializers.SerializerMethodField(required=False)
    content_preview = serializers.SerializerMethodField(required=False)

    class Meta:
        model = BlogPost
        fields = (
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
        )
        read_only_fields = (
            "id",
            "uuid",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
        )

    def get_reading_time(self, obj: BlogPost) -> int:
        if hasattr(obj, "reading_time"):
            return obj.reading_time
        body = obj.safe_translation_getter("body", any_language=True)
        if body:
            return calculate_reading_time(body)
        return 1

    def get_content_preview(self, obj: BlogPost) -> str:
        body = obj.safe_translation_getter("body", any_language=True)
        if body:
            preview = body[:200]
            return f"{preview}..." if len(body) > 200 else preview
        return ""


class BlogPostDetailSerializer(BlogPostSerializer):
    likes = PrimaryKeyRelatedField(many=True, read_only=True)
    author = serializers.SerializerMethodField()
    category = serializers.SerializerMethodField()
    tags = serializers.SerializerMethodField()
    user_has_liked = serializers.SerializerMethodField()

    class Meta(BlogPostSerializer.Meta):
        fields = (
            *BlogPostSerializer.Meta.fields,
            "likes",
            "user_has_liked",
            "seo_title",
            "seo_description",
            "seo_keywords",
        )
        read_only_fields = (
            *BlogPostSerializer.Meta.read_only_fields,
            "likes",
        )

    @extend_schema_field(
        lazy_serializer("blog.serializers.author.BlogAuthorDetailSerializer")()
    )
    def get_author(self, obj: BlogPost):
        from blog.serializers.author import BlogAuthorDetailSerializer  # noqa: PLC0415, I001

        return BlogAuthorDetailSerializer(obj.author, context=self.context).data

    @extend_schema_field(
        lazy_serializer(
            "blog.serializers.category.BlogCategoryDetailSerializer"
        )()
    )
    def get_category(self, obj: BlogPost):
        from blog.serializers.category import BlogCategoryDetailSerializer  # noqa: PLC0415, I001

        return BlogCategoryDetailSerializer(
            obj.category, context=self.context
        ).data

    @extend_schema_field(
        lazy_serializer("blog.serializers.tag.BlogTagDetailSerializer")(
            many=True
        )
    )
    def get_tags(self, obj: BlogPost):
        from blog.serializers.tag import BlogTagDetailSerializer  # noqa: PLC0415, I001

        return BlogTagDetailSerializer(
            obj.tags.all(), many=True, context=self.context
        ).data

    def get_user_has_liked(self, obj: BlogPost) -> bool:
        request = self.context.get("request")
        if request and request.user.is_authenticated:
            return obj.likes.filter(id=request.user.id).exists()
        return False


class BlogPostWriteSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[BlogPost]
):
    category = PrimaryKeyRelatedField(queryset=BlogCategory.objects.all())
    tags = PrimaryKeyRelatedField(
        queryset=BlogTag.objects.all(), many=True, required=False
    )
    author = PrimaryKeyRelatedField(queryset=BlogAuthor.objects.all())
    translations = TranslatedFieldsFieldExtend(shared_model=BlogPost)

    class Meta:
        model = BlogPost
        fields = (
            "translations",
            "slug",
            "category",
            "tags",
            "author",
            "featured",
            "is_published",
            "seo_title",
            "seo_description",
            "seo_keywords",
        )

    def validate_slug(self, value: str) -> str:
        if not value:
            raise serializers.ValidationError(_("Slug is required."))

        queryset = BlogPost.objects.filter(slug=value)
        if self.instance:
            queryset = queryset.exclude(pk=self.instance.pk)

        if queryset.exists():
            raise serializers.ValidationError(
                _("A post with this slug already exists.")
            )

        return value

    def validate_tags(self, value: list) -> list:
        if not value:
            raise serializers.ValidationError(
                _("At least one tag is required.")
            )

        if len(value) > 10:
            raise serializers.ValidationError(
                _("Maximum 10 tags allowed per post.")
            )

        return value


class BlogPostLikedPostsRequestSerializer(serializers.Serializer):
    post_ids = serializers.ListField(
        child=serializers.IntegerField(),
        help_text=_("List of post IDs to check for likes"),
    )


class BlogPostLikedPostsResponseSerializer(serializers.Serializer):
    post_ids = serializers.ListField(
        child=serializers.IntegerField(), help_text=_("List of liked post IDs")
    )



from django.contrib import admin
from django.db.models import Count
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from unfold.admin import ModelAdmin
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateTimeFilter,
    RangeNumericFilter,
    SliderNumericFilter,
)

from vat.models import Vat


class VatRangeFilter(DropdownFilter):
    title = _("VAT Range")
    parameter_name = "vat_range"

    def lookups(self, request, model_admin):
        return [
            ("zero", _("Zero VAT (0%)")),
            ("low", _("Low VAT (0.1% - 10%)")),
            ("standard", _("Standard VAT (10% - 25%)")),
            ("high", _("High VAT (25% - 50%)")),
            ("very_high", _("Very High VAT (50%+)")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "zero":
            return queryset.filter(value=0)
        elif self.value() == "low":
            return queryset.filter(value__gt=0, value__lte=10)
        elif self.value() == "standard":
            return queryset.filter(value__gt=10, value__lte=25)
        elif self.value() == "high":
            return queryset.filter(value__gt=25, value__lte=50)
        elif self.value() == "very_high":
            return queryset.filter(value__gt=50)
        return queryset


class VatUsageFilter(DropdownFilter):
    title = _("Usage Status")
    parameter_name = "usage_status"

    def lookups(self, request, model_admin):
        return [
            ("in_use", _("Currently in Use")),
            ("unused", _("Not in Use")),
            ("popular", _("Popular (5+ products)")),
            ("rare", _("Rarely Used (1-2 products)")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "in_use":
            return queryset.filter(products__isnull=False).distinct()
        elif self.value() == "unused":
            return queryset.filter(products__isnull=True)
        elif self.value() == "popular":
            return queryset.annotate(product_count=Count("products")).filter(
                product_count__gte=5
            )
        elif self.value() == "rare":
            return queryset.annotate(product_count=Count("products")).filter(
                product_count__range=(1, 2)
            )
        return queryset


@admin.register(Vat)
class VatAdmin(ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = False
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "vat_display",
        "vat_category_badge",
        "usage_metrics",
        "calculation_preview",
        "created_display",
        "updated_display",
    ]

    list_filter = [
        VatRangeFilter,
        VatUsageFilter,
        ("value", SliderNumericFilter),
        ("value", RangeNumericFilter),
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
    ]

    search_fields = ["value", "id"]

    readonly_fields = [
        "id",
        "uuid",
        "created_at",
        "updated_at",
        "usage_analytics",
        "calculation_examples",
        "products_using_vat",
    ]

    ordering = ["value"]
    list_per_page = 25

    fieldsets = (
        (
            _("VAT Configuration"),
            {
                "fields": ("value",),
                "classes": ("wide",),
                "description": _("Set the VAT percentage rate (0.0% - 100.0%)"),
            },
        ),
        (
            _("Usage Analytics"),
            {
                "fields": ("usage_analytics", "products_using_vat"),
                "classes": ("collapse",),
            },
        ),
        (
            _("Calculation Examples"),
            {
                "fields": ("calculation_examples",),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.annotate(products_count=Count("products", distinct=True))

    def vat_display(self, obj):
        value = obj.value
        formatted_value = f"{value:g}%"

        if value == 0:
            color_class = "text-green-600 dark:text-green-400"
            icon = "🆓"
        elif value <= 10:
            color_class = "text-blue-600 dark:text-blue-400"
            icon = "📉"
        elif value <= 25:
            color_class = "text-yellow-600 dark:text-yellow-400"
            icon = "📊"
        elif value <= 50:
            color_class = "text-orange-600 dark:text-orange-400"
            icon = "📈"
        else:
            color_class = "text-red-600 dark:text-red-400"
            icon = "🔥"

        safe_icon = conditional_escape(icon)
        safe_value = conditional_escape(formatted_value)
        safe_color_class = conditional_escape(color_class)

        html = (
            f'<div class="flex items-center gap-2">'
            f'<span class="text-lg">{safe_icon}</span>'
            f'<div class="text-lg font-bold {safe_color_class}">{safe_value}</div>'
            f"</div>"
        )
        return mark_safe(html)

    vat_display.short_description = _("VAT Rate")

    def vat_category_badge(self, obj):
        value = obj.value

        if value == 0:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 "
                'rounded-full gap-1">'
                "<span>🆓</span><span>Tax Free</span>"
                "</span>"
            )
        elif value <= 5:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 "
                'rounded-full gap-1">'
                "<span>📉</span><span>Reduced</span>"
                "</span>"
            )
        elif value <= 15:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-indigo-50 dark:bg-indigo-900 text-indigo-700 dark:text-indigo-300 "
                'rounded-full gap-1">'
                "<span>📊</span><span>Low</span>"
                "</span>"
            )
        elif value <= 25:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-300 "
                'rounded-full gap-1">'
                "<span>⚖️</span><span>Standard</span>"
                "</span>"
            )
        elif value <= 35:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300 "
                'rounded-full gap-1">'
                "<span>📈</span><span>High</span>"
                "</span>"
            )
        else:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 "
                'rounded-full gap-1">'
                "<span>🔥</span><span>Premium</span>"
                "</span>"
            )

    vat_category_badge.short_description = _("Category")

    def usage_metrics(self, obj):
        products_count = getattr(obj, "products_count", 0)

        if products_count == 0:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                "bg-gray-50 dark:bg-gray-900 text-gray-700 dark:text-gray-300 "
                'rounded-full">'
                "📦 Unused"
                "</span>"
            )

        safe_count = conditional_escape(str(products_count))

        if products_count <= 2:
            html = (
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f"bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-300 "
                f'rounded-full">'
                f"📊 {safe_count} products"
                f"</span>"
            )
        elif products_count <= 10:
            html = (
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f"bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 "
                f'rounded-full">'
                f"📈 {safe_count} products"
                f"</span>"
            )
        else:
            html = (
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f"bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 "
                f'rounded-full">'
                f"🔥 {safe_count} products"
                f"</span>"
            )

        return mark_safe(html)

    usage_metrics.short_description = _("Usage")

    def calculation_preview(self, obj):
        base_prices = [10, 50, 100]
        examples = []
        for price in base_prices:
            vat_amount = (price * obj.value) / 100
            total = price + vat_amount
            examples.append(f"€{price} → €{total:.2f}")

        safe_examples = conditional_escape(" | ".join(examples))
        html = (
            f'<div class="text-xs text-gray-600 dark:text-gray-400">'
            f'<div class="font-mono">{safe_examples}</div>'
            f"</div>"
        )
        return mark_safe(html)

    calculation_preview.short_description = _("Price Examples")

    def created_display(self, obj):
        date_str = obj.created_at.strftime("%Y-%m-%d")
        safe_date = conditional_escape(date_str)
        return mark_safe(
            f'<div class="text-sm text-gray-600 dark:text-gray-400">{safe_date}</div>'
        )

    created_display.short_description = _("Created")

    def updated_display(self, obj):
        date_str = obj.updated_at.strftime("%Y-%m-%d")
        safe_date = conditional_escape(date_str)
        return mark_safe(
            f'<div class="text-sm text-gray-600 dark:text-gray-400">{safe_date}</div>'
        )

    updated_display.short_description = _("Updated")

    def usage_analytics(self, obj):
        try:
            products_count = getattr(obj, "products_count", 0)
            base_calculation = 1000
            vat_amount = (base_calculation * obj.value) / 100
            total_with_vat = base_calculation + vat_amount

            vat_formatted = f"€{vat_amount:.2f}"
            total_formatted = f"€{total_with_vat:.2f}"
            value_formatted = f"{obj.value:g}%"
            category_text = self._get_vat_category_text(obj.value)
            status_text = "In Use" if products_count > 0 else "Unused"

            safe_products = conditional_escape(str(products_count))
            safe_value = conditional_escape(value_formatted)
            safe_vat = conditional_escape(vat_formatted)
            safe_total = conditional_escape(total_formatted)
            safe_category = conditional_escape(category_text)
            safe_status = conditional_escape(status_text)

            html = (
                f'<div class="text-sm">'
                f'<div class="grid grid-cols-2 gap-2">'
                f"<div><strong>Products Using:</strong></div><div>{safe_products}</div>"
                f"<div><strong>VAT Rate:</strong></div><div>{safe_value}</div>"
                f"<div><strong>Sample VAT on €1000:</strong></div><div>{safe_vat}</div>"
                f"<div><strong>Total with VAT:</strong></div><div>{safe_total}</div>"
                f"<div><strong>Category:</strong></div><div>{safe_category}</div>"
                f"<div><strong>Status:</strong></div><div>{safe_status}</div>"
                f"</div>"
                f"</div>"
            )
            return mark_safe(html)
        except Exception:
            return mark_safe(
                '<span class="text-gray-500">Data unavailable</span>'
            )

    usage_analytics.short_description = _("Usage Analytics")

    def calculation_examples(self, obj):
        examples = []
        base_amounts = [10, 25, 50, 100, 250, 500, 1000]

        for amount in base_amounts:
            vat_amount = (amount * obj.value) / 100
            total = amount + vat_amount
            examples.append(f"€{amount:g} + VAT = €{total:.2f}")

        safe_items = "".join(
            f"<div>{conditional_escape(example)}</div>" for example in examples
        )
        html = (
            f'<div class="text-sm">'
            f'<div class="grid grid-cols-2 gap-2">'
            f"{safe_items}"
            f"</div>"
            f"</div>"
        )
        return mark_safe(html)

    calculation_examples.short_description = _("Calculation Examples")

    def products_using_vat(self, obj):
        try:
            products_count = getattr(obj, "products_count", 0)

            if products_count == 0:
                return mark_safe(
                    '<div class="text-sm text-gray-500 dark:text-gray-400 italic">'
                    "No products currently using this VAT rate"
                    "</div>"
                )

            safe_count = conditional_escape(str(products_count))
            html = (
                f'<div class="text-sm">'
                f'<div class="font-medium text-gray-900 dark:text-gray-100">'
                f"{safe_count} products using this VAT rate"
                f"</div>"
                f'<div class="text-gray-500 dark:text-gray-400 mt-1">'
                f"Click to view products →"
                f"</div>"
                f"</div>"
            )
            return mark_safe(html)
        except Exception:
            return mark_safe(
                '<span class="text-gray-500">Data unavailable</span>'
            )

    products_using_vat.short_description = _("Products Using This VAT")

    def _get_vat_category_text(self, value):
        if value == 0:
            return "Tax Free"
        elif value <= 5:
            return "Reduced Rate"
        elif value <= 15:
            return "Low Rate"
        elif value <= 25:
            return "Standard Rate"
        elif value <= 35:
            return "High Rate"
        else:
            return "Premium Rate"



# Generated by Django 4.2.9 on 2024-02-10 14:39
import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("product", "0001_initial"),
        ("pay_way", "0001_initial"),
        ("region", "0001_initial"),
        ("order", "0001_initial"),
        ("country", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="orderitem",
            name="product",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="order_item_product",
                to="product.product",
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="country",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="order_country",
                to="country.country",
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="pay_way",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="order_pay_way",
                to="pay_way.payway",
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="region",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="order_region",
                to="region.region",
            ),
        ),
    ]



from datetime import timedelta
from unittest.mock import Mock

from django.contrib.admin.sites import AdminSite
from django.contrib.auth import get_user_model
from django.test import RequestFactory, TestCase
from django.utils import timezone

from notification.admin import (
    NotificationAdmin,
    NotificationStatusFilter,
    NotificationUserAdmin,
    NotificationUserStatusFilter,
)
from notification.models.notification import Notification
from notification.models.user import NotificationUser

User = get_user_model()


class NotificationStatusFilterTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = Mock()

        self.model_admin = Mock()
        self.filter = NotificationStatusFilter(
            self.request, {}, Notification, self.model_admin
        )

        self.notification_active = Notification.objects.create(
            title="Active Notification",
            message="Active message",
            kind="INFO",
            category="SYSTEM",
            priority="NORMAL",
            notification_type="general",
            expiry_date=timezone.now() + timedelta(days=7),
        )

        self.notification_expired = Notification.objects.create(
            title="Expired Notification",
            message="Expired message",
            kind="WARNING",
            category="ORDER",
            priority="HIGH",
            notification_type="order_status",
            expiry_date=timezone.now() - timedelta(days=1),
        )

        self.notification_urgent = Notification.objects.create(
            title="Urgent Notification",
            message="Urgent message",
            kind="ERROR",
            category="SECURITY",
            priority="URGENT",
            notification_type="security_alert",
        )

        self.notification_with_link = Notification.objects.create(
            title="Notification with Link",
            message="Link message",
            kind="INFO",
            category="PROMOTION",
            priority="NORMAL",
            notification_type="promotion",
            link="https://example.com",
        )

    def test_lookups(self):
        lookups = self.filter.lookups(self.request, self.model_admin)

        self.assertEqual(len(lookups), 10)
        self.assertIn(("active", "Active"), lookups)
        self.assertIn(("expired", "Expired"), lookups)
        self.assertIn(("urgent", "Urgent Priority"), lookups)
        self.assertIn(("recent", "Recent (Last 7 days)"), lookups)
        self.assertIn(("with_link", "Has Link"), lookups)
        self.assertIn(("system", "System Notifications"), lookups)
        self.assertIn(("order", "Order Related"), lookups)
        self.assertIn(("payment", "Payment Related"), lookups)
        self.assertIn(("security", "Security Related"), lookups)
        self.assertIn(("promotion", "Promotions"), lookups)

    def test_queryset_active(self):
        self.filter.used_parameters = {"notification_status": "active"}
        queryset = Notification.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_active, result)
        self.assertNotIn(self.notification_expired, result)

    def test_queryset_expired(self):
        self.filter.used_parameters = {"notification_status": "expired"}
        queryset = Notification.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_expired, result)
        self.assertNotIn(self.notification_active, result)

    def test_queryset_urgent(self):
        self.filter.used_parameters = {"notification_status": "urgent"}
        queryset = Notification.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_urgent, result)
        self.assertNotIn(self.notification_active, result)

    def test_queryset_with_link(self):
        self.filter.used_parameters = {"notification_status": "with_link"}
        queryset = Notification.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_with_link, result)
        self.assertNotIn(self.notification_active, result)

    def test_queryset_by_category(self):
        test_cases = [
            ("system", self.notification_active),
            ("order", self.notification_expired),
            ("security", self.notification_urgent),
            ("promotion", self.notification_with_link),
        ]

        for filter_value, expected_notification in test_cases:
            with self.subTest(filter_value=filter_value):
                self.filter.used_parameters = {
                    "notification_status": filter_value
                }
                queryset = Notification.objects.all()

                result = self.filter.queryset(self.request, queryset)

                self.assertIn(expected_notification, result)

    def test_queryset_recent(self):
        self.filter.used_parameters = {"notification_status": "recent"}
        queryset = Notification.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertTrue(result.exists())

    def test_queryset_default(self):
        self.filter.used_parameters = {"notification_status": "invalid"}
        queryset = Notification.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertEqual(list(result), list(queryset))


class NotificationUserStatusFilterTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = Mock()

        self.model_admin = Mock()
        self.filter = NotificationUserStatusFilter(
            self.request, {}, NotificationUser, self.model_admin
        )

        self.user = User.objects.create_user(
            username="testuser",
            email="test@example.com",
            password="testpass123",
        )

        self.notification = Notification.objects.create(
            title="Test Notification",
            message="Test message",
            kind="INFO",
            category="SYSTEM",
            priority="NORMAL",
            notification_type="general",
        )

        self.notification_urgent = Notification.objects.create(
            title="Urgent Notification",
            message="Urgent message",
            kind="ERROR",
            category="SECURITY",
            priority="URGENT",
            notification_type="security_alert",
        )

        self.notification_user_seen = NotificationUser.objects.create(
            user=self.user,
            notification=self.notification,
            seen=True,
            seen_at=timezone.now(),
        )

        self.notification_user_unseen = NotificationUser.objects.create(
            user=self.user, notification=self.notification_urgent, seen=False
        )

    def test_lookups(self):
        lookups = self.filter.lookups(self.request, self.model_admin)

        self.assertEqual(len(lookups), 5)
        self.assertIn(("seen", "Seen"), lookups)
        self.assertIn(("unseen", "Unseen"), lookups)
        self.assertIn(("recent_seen", "Seen Today"), lookups)
        self.assertIn(("urgent_unseen", "Urgent & Unseen"), lookups)
        self.assertIn(("expired", "Expired Notifications"), lookups)

    def test_queryset_seen(self):
        self.filter.used_parameters = {"user_status": "seen"}
        queryset = NotificationUser.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_user_seen, result)
        self.assertNotIn(self.notification_user_unseen, result)

    def test_queryset_unseen(self):
        self.filter.used_parameters = {"user_status": "unseen"}
        queryset = NotificationUser.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_user_unseen, result)
        self.assertNotIn(self.notification_user_seen, result)

    def test_queryset_urgent_unseen(self):
        self.filter.used_parameters = {"user_status": "urgent_unseen"}
        queryset = NotificationUser.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertIn(self.notification_user_unseen, result)
        self.assertNotIn(self.notification_user_seen, result)

    def test_queryset_default(self):
        self.filter.used_parameters = {"user_status": "invalid"}
        queryset = NotificationUser.objects.all()

        result = self.filter.queryset(self.request, queryset)

        self.assertEqual(list(result), list(queryset))


class NotificationAdminTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.site = AdminSite()
        self.admin = NotificationAdmin(Notification, self.site)

        self.notification = Notification.objects.create(
            title="Test Notification",
            message="Test notification message",
            kind="INFO",
            category="SYSTEM",
            priority="HIGH",
            notification_type="general",
            link="https://example.com",
            expiry_date=timezone.now() + timedelta(days=7),
        )

        self.user = User.objects.create_user(
            username="testuser",
            email="test@example.com",
            password="testpass123",
        )

        self.notification_user = NotificationUser.objects.create(
            user=self.user,
            notification=self.notification,
            seen=True,
            seen_at=timezone.now(),
        )

    def test_get_queryset(self):
        request = self.factory.get("/")
        request.user = Mock()

        queryset = self.admin.get_queryset(request)

        self.assertTrue(hasattr(queryset, "_prefetch_related_lookups"))
        self.assertIn("user__user", queryset._prefetch_related_lookups)

    def test_notification_info(self):
        result = self.admin.notification_info(self.notification)

        self.assertIn("Test Notification", result)
        self.assertIn("Test notification message", result)
        self.assertIn("🔗", result)
        self.assertIn("ID:", result)

    def test_priority_badge(self):
        test_cases = [
            ("LOW", "inline-flex"),
            ("NORMAL", "inline-flex"),
            ("HIGH", "inline-flex"),
            ("URGENT", "inline-flex"),
            ("CRITICAL", "inline-flex"),
        ]

        for priority, expected_css in test_cases:
            with self.subTest(priority=priority):
                self.notification.priority = priority
                result = self.admin.priority_badge(self.notification)

                self.assertIn(expected_css, result)
                self.assertTrue(len(result) > 0)

    def test_category_badge(self):
        test_cases = [
            ("SYSTEM", "⚙️"),
            ("ORDER", "📦"),
            ("PAYMENT", "💳"),
            ("SECURITY", "🔐"),
            ("PROMOTION", "🎉"),
        ]

        for category, expected_emoji in test_cases:
            with self.subTest(category=category):
                self.notification.category = category
                result = self.admin.category_badge(self.notification)

                self.assertIn(expected_emoji, result)
                self.assertIn("inline-flex", result)

    def test_status_display(self):
        result = self.admin.status_display(self.notification)
        self.assertIn("Active", result)

        self.notification.expiry_date = timezone.now() - timedelta(days=1)
        result = self.admin.status_display(self.notification)
        self.assertIn("Expired", result)

        self.notification.expiry_date = None
        result = self.admin.status_display(self.notification)
        self.assertIn("Active", result)

    def test_engagement_stats(self):
        result = self.admin.engagement_stats(self.notification)

        self.assertIn("👥", result)
        self.assertIn("👁️", result)
        self.assertIn("👓", result)
        self.assertIn("%", result)

    def test_timing_info(self):
        result = self.admin.timing_info(self.notification)

        self.assertIn("ago", result)
        self.assertTrue(len(result) > 0)
        import re

        self.assertTrue(re.search(r"\d{2}-\d{2}", result))
        self.assertTrue(
            "left" in result or "No expiry" in result or "Expired" in result
        )

    def test_notification_analytics(self):
        result = self.admin.notification_analytics(self.notification)

        self.assertIn("Age:", result)
        self.assertIn("Title Length:", result)
        self.assertIn("Message Length:", result)
        self.assertIn("Has Link:", result)
        self.assertIn("Has Type:", result)
        self.assertIn("Readability:", result)
        self.assertIn("chars", result)

    def test_engagement_summary(self):
        result = self.admin.engagement_summary(self.notification)

        self.assertIn("Total Recipients:", result)
        self.assertIn("Seen:", result)
        self.assertIn("Unseen:", result)
        self.assertIn("Engagement Rate:", result)
        self.assertIn("%", result)
        self.assertIn("Performance:", result)

    def test_timing_summary(self):
        result = self.admin.timing_summary(self.notification)

        self.assertIn("Created", result)
        if self.notification.expiry_date:
            self.assertIn("Expires", result)


class NotificationUserAdminTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.site = AdminSite()
        self.admin = NotificationUserAdmin(NotificationUser, self.site)

        self.user = User.objects.create_user(
            username="testuser",
            email="test@example.com",
            password="testpass123",
        )

        self.notification = Notification.objects.create(
            title="Test Notification",
            message="Test message",
            kind="INFO",
            category="SYSTEM",
            priority="HIGH",
            notification_type="general",
        )

        self.notification_user = NotificationUser.objects.create(
            user=self.user,
            notification=self.notification,
            seen=True,
            seen_at=timezone.now(),
        )

    def test_get_queryset(self):
        request = self.factory.get("/")
        request.user = Mock()

        queryset = self.admin.get_queryset(request)

        self.assertTrue(
            hasattr(queryset, "_prefetch_related_lookups")
            or hasattr(queryset, "query")
        )

    def test_user_info(self):
        result = self.admin.user_info(self.notification_user)

        self.assertIn("testuser", result)
        self.assertIn("test@example.com", result)

    def test_notification_info(self):
        result = self.admin.notification_info(self.notification_user)

        self.assertIn("Test Notification", result)
        self.assertTrue(len(result) > 0)
        self.assertIn("#", result)

    def test_seen_status(self):
        result = self.admin.seen_status(self.notification_user)
        self.assertIn("Seen", result)

        self.notification_user.seen = False
        self.notification_user.seen_at = None
        result = self.admin.seen_status(self.notification_user)
        self.assertIn("Unseen", result)

    def test_priority_indicator(self):
        result = self.admin.priority_indicator(self.notification_user)

        self.assertIn("⚠️", result)
        self.assertIn("text-orange-600", result)
        self.assertTrue(len(result) > 0)

    def test_timing_display(self):
        result = self.admin.timing_display(self.notification_user)

        self.assertIn("ago", result)
        self.assertTrue(len(result) > 0)
        import re

        self.assertTrue(re.search(r"\d{2}-\d{2}", result))
        self.assertTrue("Active" in result or "Expired" in result)

    def test_user_notification_analytics(self):
        result = self.admin.user_notification_analytics(self.notification_user)

        self.assertIn("Age:", result)
        self.assertIn("Response Time:", result)
        self.assertIn("Notification Status:", result)
        self.assertIn("Priority Level:", result)
        self.assertIn("Category:", result)
        self.assertIn("Engagement:", result)


class NotificationAdminIntegrationTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.site = AdminSite()
        self.notification_admin = NotificationAdmin(Notification, self.site)
        self.notification_user_admin = NotificationUserAdmin(
            NotificationUser, self.site
        )

        self.user = User.objects.create_user(
            username="adminuser",
            email="admin@example.com",
            password="adminpass123",
        )

        self.notifications = []
        priorities = ["LOW", "NORMAL", "HIGH", "URGENT", "CRITICAL"]
        categories = ["SYSTEM", "ORDER", "PAYMENT", "SECURITY", "PROMOTION"]

        for i, (priority, category) in enumerate(zip(priorities, categories)):
            notification = Notification.objects.create(
                title=f"Test Notification {i + 1}",
                message=f"Test message {i + 1}",
                kind="INFO",
                category=category,
                priority=priority,
                notification_type="general",
            )
            self.notifications.append(notification)

            NotificationUser.objects.create(
                user=self.user,
                notification=notification,
                seen=i % 2 == 0,
            )

    def test_admin_display_methods_integration(self):
        request = self.factory.get("/")
        request.user = Mock()

        for notification in self.notifications:
            notification_info = self.notification_admin.notification_info(
                notification
            )
            priority_badge = self.notification_admin.priority_badge(
                notification
            )
            category_badge = self.notification_admin.category_badge(
                notification
            )
            status_display = self.notification_admin.status_display(
                notification
            )

            self.assertIsInstance(notification_info, str)
            self.assertIsInstance(priority_badge, str)
            self.assertIsInstance(category_badge, str)
            self.assertIsInstance(status_display, str)

            self.assertIn(notification.title, notification_info)
            self.assertIn("inline-flex", priority_badge)
            self.assertIn("inline-flex", category_badge)

    def test_filter_functionality(self):
        request = self.factory.get("/")
        request.user = Mock()

        status_filter = NotificationStatusFilter(
            request, {}, Notification, self.notification_admin
        )

        filter_tests = [
            ("urgent", lambda n: n.priority in ["URGENT", "CRITICAL"]),
            ("system", lambda n: n.category == "SYSTEM"),
            ("order", lambda n: n.category == "ORDER"),
            ("payment", lambda n: n.category == "PAYMENT"),
        ]

        for filter_value, condition_func in filter_tests:
            status_filter.used_parameters = {
                "notification_status": filter_value
            }
            filtered_queryset = status_filter.queryset(
                request, Notification.objects.all()
            )

            for notification in filtered_queryset:
                self.assertTrue(condition_func(notification))

    def test_queryset_optimization(self):
        request = self.factory.get("/")
        request.user = Mock()

        notification_queryset = self.notification_admin.get_queryset(request)
        self.assertTrue(
            hasattr(notification_queryset, "_prefetch_related_lookups")
        )

        user_queryset = self.notification_user_admin.get_queryset(request)
        self.assertTrue(hasattr(user_queryset, "query"))



import math

from rest_framework import pagination
from rest_framework.response import Response


class CursorPaginator(pagination.CursorPagination):
    page_size = 20
    max_page_size = 100
    page_size_query_param = "page_size"
    cursor_query_param = "cursor"
    total_items = 0
    total_pages = 0
    ordering = "-created_at"
    has_next = False
    has_previous = False

    def paginate_queryset(self, queryset, request, view=None):
        page_size_param = request.query_params.get(
            self.page_size_query_param
        ) or request.query_params.get("page_size")
        if page_size_param:
            try:
                self.page_size = min(int(page_size_param), self.max_page_size)
            except (ValueError, TypeError):
                pass

        self.total_items = queryset.count()

        page = super().paginate_queryset(queryset, request, view)
        self.page = page

        self.total_pages = self.get_total_pages()

        return page

    def get_total_pages(self):
        if self.total_items == 0:
            return 1
        actual_page_size = getattr(self, "page_size", 20)
        return math.ceil(self.total_items / actual_page_size)

    def get_paginated_response(self, data):
        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.total_items,
                "total_pages": self.total_pages,
                "page_size": self.page_size,
                "page_total_results": len(data),
                "results": data,
            }
        )

    def get_paginated_response_schema(self, schema):
        return {
            "type": "object",
            "required": ["count", "results"],
            "properties": {
                "links": {
                    "type": "object",
                    "properties": {
                        "next": {
                            "type": "string",
                            "nullable": True,
                            "format": "uri",
                        },
                        "previous": {
                            "type": "string",
                            "nullable": True,
                            "format": "uri",
                        },
                    },
                },
                "count": {"type": "integer", "example": 123},
                "total_pages": {"type": "integer", "example": 123},
                "page_size": {"type": "integer", "example": 123},
                "page_total_results": {"type": "integer", "example": 123},
                "results": schema,
            },
        }



import datetime
from unittest.mock import patch

from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.db import models
from django.test import RequestFactory, TestCase, override_settings
from django.utils import timezone
from unfold.contrib.filters.admin import RangeDateTimeFilter
from zoneinfo import ZoneInfo

from contact.admin import ContactAdmin, MessageLengthFilter, RecentContactFilter
from contact.models import Contact


class TestMessageLengthFilter(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = User(username="testuser")

        class MockModelAdmin:
            pass

        self.model_admin = MockModelAdmin()

        self.short_contact = Contact.objects.create(
            name="Short User",
            email="short@example.com",
            message="Short message",
        )

        self.medium_contact = Contact.objects.create(
            name="Medium User",
            email="medium@example.com",
            message="A" * 200,
        )

        self.long_contact = Contact.objects.create(
            name="Long User",
            email="long@example.com",
            message="A" * 600,
        )

    def test_filter_title_and_parameter(self):
        filter_instance = MessageLengthFilter(
            self.request, {}, Contact, self.model_admin
        )
        self.assertEqual(filter_instance.title, "Message Length")
        self.assertEqual(filter_instance.parameter_name, "message_length")

    def test_lookups(self):
        filter_instance = MessageLengthFilter(
            self.request, {}, Contact, self.model_admin
        )

        lookups = filter_instance.lookups(self.request, None)
        expected_lookups = [
            ("short", "Short (<100 chars)"),
            ("medium", "Medium (100-500 chars)"),
            ("long", "Long (>500 chars)"),
        ]

        self.assertEqual(list(lookups), expected_lookups)

    def test_queryset_short_filter(self):
        request = self.factory.get("/", {"message_length": "short"})
        request.user = self.request.user
        filter_instance = MessageLengthFilter(
            request, request.GET.copy(), Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filtered_queryset = filter_instance.queryset(request, queryset)

        self.assertIn(self.short_contact, filtered_queryset)
        self.assertNotIn(self.medium_contact, filtered_queryset)
        self.assertNotIn(self.long_contact, filtered_queryset)

    def test_queryset_medium_filter(self):
        request = self.factory.get("/", {"message_length": "medium"})
        request.user = self.request.user
        filter_instance = MessageLengthFilter(
            request, request.GET.copy(), Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filtered_queryset = filter_instance.queryset(request, queryset)

        self.assertNotIn(self.short_contact, filtered_queryset)
        self.assertIn(self.medium_contact, filtered_queryset)
        self.assertNotIn(self.long_contact, filtered_queryset)

    def test_queryset_long_filter(self):
        request = self.factory.get("/", {"message_length": "long"})
        request.user = self.request.user

        filter_instance = MessageLengthFilter(
            request, request.GET.copy(), Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filtered_queryset = filter_instance.queryset(request, queryset)

        self.assertNotIn(self.short_contact, filtered_queryset)
        self.assertNotIn(self.medium_contact, filtered_queryset)
        self.assertIn(self.long_contact, filtered_queryset)

    def test_queryset_no_filter(self):
        filter_instance = MessageLengthFilter(
            self.request, {}, Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filtered_queryset = filter_instance.queryset(self.request, queryset)

        self.assertEqual(list(queryset), list(filtered_queryset))

    def test_queryset_invalid_filter(self):
        request = self.factory.get("/", {"message_length": "invalid"})
        request.user = self.request.user
        filter_instance = MessageLengthFilter(
            request, request.GET.copy(), Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filtered_queryset = filter_instance.queryset(request, queryset)

        self.assertEqual(list(queryset), list(filtered_queryset))


@override_settings(LANGUAGE_CODE="en-US")
class TestRecentContactFilter(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = User(username="testuser")

        class MockModelAdmin:
            pass

        self.model_admin = MockModelAdmin()

        now = timezone.now()

        self.today_contact = Contact.objects.create(
            name="Today User",
            email="today@example.com",
            message="Today message",
        )
        self.today_contact.created_at = now
        self.today_contact.save()

        self.week_contact = Contact.objects.create(
            name="Week User", email="week@example.com", message="Week message"
        )
        self.week_contact.created_at = now - timezone.timedelta(days=3)
        self.week_contact.save()

        self.month_contact = Contact.objects.create(
            name="Month User",
            email="month@example.com",
            message="Month message",
        )
        self.month_contact.created_at = now - timezone.timedelta(days=15)
        self.month_contact.save()

        self.quarter_contact = Contact.objects.create(
            name="Quarter User",
            email="quarter@example.com",
            message="Quarter message",
        )
        self.quarter_contact.created_at = now - timezone.timedelta(days=60)
        self.quarter_contact.save()

        self.old_contact = Contact.objects.create(
            name="Old User", email="old@example.com", message="Old message"
        )
        self.old_contact.created_at = now - timezone.timedelta(days=200)
        self.old_contact.save()

    def test_filter_title_and_parameter(self):
        filter_instance = RecentContactFilter(
            self.request, {}, Contact, self.model_admin
        )
        self.assertEqual(filter_instance.title, "Contact Period")
        self.assertEqual(filter_instance.parameter_name, "contact_period")

    def test_lookups(self):
        filter_instance = RecentContactFilter(
            self.request, {}, Contact, self.model_admin
        )

        lookups = filter_instance.lookups(self.request, None)
        expected_lookups = [
            ("today", "Today"),
            ("week", "This Week"),
            ("month", "This Month"),
            ("quarter", "This Quarter"),
        ]

        self.assertEqual(list(lookups), expected_lookups)

    @patch("contact.admin.timezone")
    def test_queryset_today_filter(self, mock_timezone):
        mock_now = datetime.datetime(
            2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC")
        )
        mock_timezone.now.return_value = mock_now

        filter_instance = RecentContactFilter(
            self.request, {"contact_period": "today"}, Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filter_instance.queryset(self.request, queryset)

        mock_timezone.now.assert_called()

    @patch("contact.admin.timezone")
    def test_queryset_week_filter(self, mock_timezone):
        mock_now = datetime.datetime(
            2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC")
        )
        mock_timezone.now.return_value = mock_now

        filter_instance = RecentContactFilter(
            self.request, {"contact_period": "week"}, Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filter_instance.queryset(self.request, queryset)

        mock_timezone.now.assert_called()

    @patch("contact.admin.timezone")
    def test_queryset_month_filter(self, mock_timezone):
        mock_now = datetime.datetime(
            2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC")
        )
        mock_timezone.now.return_value = mock_now

        filter_instance = RecentContactFilter(
            self.request, {"contact_period": "month"}, Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filter_instance.queryset(self.request, queryset)

        mock_timezone.now.assert_called()

    @patch("contact.admin.timezone")
    def test_queryset_quarter_filter(self, mock_timezone):
        mock_now = datetime.datetime(
            2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC")
        )
        mock_timezone.now.return_value = mock_now

        filter_instance = RecentContactFilter(
            self.request,
            {"contact_period": "quarter"},
            Contact,
            self.model_admin,
        )

        queryset = Contact.objects.all()
        filter_instance.queryset(self.request, queryset)

        mock_timezone.now.assert_called()

    def test_queryset_no_filter(self):
        filter_instance = RecentContactFilter(
            self.request, {}, Contact, self.model_admin
        )

        queryset = Contact.objects.all()
        filtered_queryset = filter_instance.queryset(self.request, queryset)

        self.assertEqual(list(queryset), list(filtered_queryset))


class TestContactAdmin(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = User(username="testuser")

        self.site = AdminSite()
        self.admin = ContactAdmin(Contact, self.site)

        self.contact = Contact.objects.create(
            name="John Doe Test User",
            email="john.doe@example.com",
            message="This is a test message with some content to analyze.",
        )

    def test_admin_configuration(self):
        self.assertTrue(self.admin.compressed_fields)
        self.assertTrue(self.admin.warn_unsaved_form)
        self.assertTrue(self.admin.list_fullwidth)
        self.assertTrue(self.admin.list_filter_submit)
        self.assertTrue(self.admin.list_filter_sheet)
        self.assertEqual(self.admin.list_per_page, 25)
        self.assertEqual(self.admin.date_hierarchy, "created_at")

    def test_list_display(self):
        expected_display = [
            "contact_info",
            "message_preview",
            "message_stats",
            "contact_timing",
            "priority_badge",
        ]
        self.assertEqual(self.admin.list_display, expected_display)

    def test_search_fields(self):
        expected_fields = ["name", "email", "message"]
        self.assertEqual(self.admin.search_fields, expected_fields)

    def test_readonly_fields(self):
        expected_fields = (
            "id",
            "uuid",
            "created_at",
            "updated_at",
            "contact_analytics",
            "message_analytics",
            "timing_info",
        )
        self.assertEqual(self.admin.readonly_fields, expected_fields)

    def test_get_queryset(self):
        queryset = self.admin.get_queryset(self.request)
        self.assertIsInstance(queryset, models.QuerySet)
        self.assertIn(self.contact, queryset)

    def test_get_ordering(self):
        ordering = self.admin.get_ordering(self.request)
        expected = ["-created_at", "name"]
        self.assertEqual(ordering, expected)

    def test_contact_info_normal_email(self):
        result = self.admin.contact_info(self.contact)

        self.assertIn("John Doe Test User", result)
        self.assertIn("john.doe@example.com", result)
        self.assertIn(str(self.contact.id), result)
        self.assertIn("text-base-600", result)

    def test_contact_info_suspicious_email(self):
        suspicious_contact = Contact.objects.create(
            name="Suspicious User", email="invalidemail", message="Test message"
        )

        result = self.admin.contact_info(suspicious_contact)

        self.assertIn("text-red-600", result)
        self.assertIn("invalidemail", result)

    def test_contact_info_long_name(self):
        long_name_contact = Contact.objects.create(
            name="A" * 50,
            email="test@example.com",
            message="Test message",
        )

        result = self.admin.contact_info(long_name_contact)

        self.assertIn("...", result)
        self.assertNotIn("A" * 50, result)

    def test_message_preview_short_message(self):
        result = self.admin.message_preview(self.contact)

        self.assertIn("This is a test message", result)
        self.assertNotIn("...", result)

    def test_message_preview_long_message(self):
        long_message_contact = Contact.objects.create(
            name="Long Message User",
            email="long@example.com",
            message="A" * 150,
        )

        result = self.admin.message_preview(long_message_contact)

        self.assertIn("...", result)

    def test_message_preview_newlines(self):
        newline_contact = Contact.objects.create(
            name="Newline User",
            email="newline@example.com",
            message="Line 1\nLine 2\nLine 3",
        )

        result = self.admin.message_preview(newline_contact)

        self.assertIn("Line 1 Line 2 Line 3", result)

    def test_message_stats_short_message(self):
        result = self.admin.message_stats(self.contact)

        self.assertIn("text-green-700", result)
        self.assertIn("chars", result)
        self.assertIn("words", result)

    def test_message_stats_medium_message(self):
        medium_contact = Contact.objects.create(
            name="Medium User",
            email="medium@example.com",
            message="A" * 300,
        )

        result = self.admin.message_stats(medium_contact)

        self.assertIn("text-yellow-700", result)

    def test_message_stats_long_message(self):
        long_contact = Contact.objects.create(
            name="Long User",
            email="long@example.com",
            message="A" * 600,
        )

        result = self.admin.message_stats(long_contact)

        self.assertIn("text-blue-700", result)

    @patch("contact.admin.timezone")
    def test_contact_timing_just_now(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        self.contact.created_at = now - timezone.timedelta(minutes=5)
        self.contact.save()

        result = self.admin.contact_timing(self.contact)
        self.assertIn("Just Now", result)
        self.assertIn("text-red-700", result)

    @patch("contact.admin.timezone")
    def test_contact_timing_hours_ago(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        self.contact.created_at = now - timezone.timedelta(hours=3)
        self.contact.save()

        result = self.admin.contact_timing(self.contact)
        self.assertIn("3h ago", result)
        self.assertIn("text-orange-700", result)

    @patch("contact.admin.timezone")
    def test_contact_timing_days_ago(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        self.contact.created_at = now - timezone.timedelta(days=2)
        self.contact.save()

        result = self.admin.contact_timing(self.contact)
        self.assertIn("2d ago", result)

    @patch("contact.admin.timezone")
    def test_contact_timing_old(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        self.contact.created_at = now - timezone.timedelta(days=10)
        self.contact.save()

        result = self.admin.contact_timing(self.contact)
        self.assertIn("Old", result)
        self.assertIn("text-base-700", result)

    @patch("contact.admin.timezone")
    def test_priority_badge_urgent(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        urgent_contact = Contact.objects.create(
            name="Urgent User",
            email="urgent@example.com",
            message="URGENT: Need immediate help with billing issue!",
        )
        urgent_contact.created_at = now - timezone.timedelta(minutes=5)
        urgent_contact.save()

        result = self.admin.priority_badge(urgent_contact)
        self.assertIn("Urgent", result)
        self.assertIn("bg-red-50", result)

    @patch("contact.admin.timezone")
    def test_priority_badge_high(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        high_contact = Contact.objects.create(
            name="High User",
            email="high@example.com",
            message="A" * 600,
        )
        high_contact.created_at = now - timezone.timedelta(hours=2)
        high_contact.save()

        result = self.admin.priority_badge(high_contact)
        self.assertIn("High", result)
        self.assertIn("bg-orange-50", result)

    @patch("contact.admin.timezone")
    def test_priority_badge_medium(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        medium_contact = Contact.objects.create(
            name="Medium User",
            email="medium@example.com",
            message="General inquiry about services",
        )
        medium_contact.created_at = now - timezone.timedelta(hours=12)
        medium_contact.save()

        result = self.admin.priority_badge(medium_contact)
        self.assertIn("Medium", result)
        self.assertIn("bg-yellow-50", result)

    @patch("contact.admin.timezone")
    def test_priority_badge_low(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        low_contact = Contact.objects.create(
            name="Low User",
            email="low@example.com",
            message="Just wanted to say thanks",
        )
        low_contact.created_at = now - timezone.timedelta(days=10)
        low_contact.save()

        result = self.admin.priority_badge(low_contact)
        self.assertIn("Low", result)
        self.assertIn("bg-green-50", result)

    @patch("contact.admin.timezone")
    def test_contact_analytics(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        self.contact.created_at = now - timezone.timedelta(hours=3)
        self.contact.save()

        result = self.admin.contact_analytics(self.contact)

        self.assertIn("Response Time", result)
        self.assertIn("Priority Score", result)

    def test_message_analytics(self):
        result = self.admin.message_analytics(self.contact)

        self.assertIn("Reading Time", result)
        self.assertIn("Sentiment", result)
        self.assertIn("Complexity", result)
        self.assertIn("Language", result)

        self.assertIn("seconds", result)
        self.assertIn("Neutral", result)
        self.assertIn("English", result)

    def test_message_analytics_urgent_words(self):
        urgent_contact = Contact.objects.create(
            name="Urgent User",
            email="urgent@example.com",
            message="URGENT help needed ASAP emergency situation!",
        )

        result = self.admin.message_analytics(urgent_contact)

        self.assertIn("Urgent", result)

    @patch("contact.admin.timezone")
    def test_timing_info(self, mock_timezone):
        now = datetime.datetime(2023, 12, 15, 14, 30, 0, tzinfo=ZoneInfo("UTC"))
        mock_timezone.now.return_value = now

        created_time = datetime.datetime(
            2023, 12, 13, 10, 15, 0, tzinfo=ZoneInfo("UTC")
        )
        self.contact.created_at = created_time
        self.contact.save()

        result = self.admin.timing_info(self.contact)

        self.assertIn("Age", result)
        self.assertIn("Business Hours", result)
        self.assertIn("Season", result)

        self.assertIn("Yes", result)

    def test_timing_info_business_hours(self):
        business_time = datetime.datetime(
            2023, 12, 13, 10, 0, 0, tzinfo=ZoneInfo("UTC")
        )
        self.contact.created_at = business_time
        self.contact.save()

        result = self.admin.timing_info(self.contact)
        self.assertIn("Yes", result)

    def test_timing_info_weekend(self):
        weekend_time = datetime.datetime(
            2023, 12, 16, 10, 0, 0, tzinfo=ZoneInfo("UTC")
        )
        self.contact.created_at = weekend_time
        self.contact.save()

        result = self.admin.timing_info(self.contact)
        self.assertIn("Weekend Contact", result)

    def test_get_season_winter(self):
        winter_date = datetime.date(2023, 1, 15)
        season = self.admin._get_season(winter_date)
        self.assertEqual(season, "Winter")

    def test_get_season_spring(self):
        spring_date = datetime.date(2023, 4, 15)
        season = self.admin._get_season(spring_date)
        self.assertEqual(season, "Spring")

    def test_get_season_summer(self):
        summer_date = datetime.date(2023, 7, 15)
        season = self.admin._get_season(summer_date)
        self.assertEqual(season, "Summer")

    def test_get_season_autumn(self):
        autumn_date = datetime.date(2023, 10, 15)
        season = self.admin._get_season(autumn_date)
        self.assertEqual(season, "Autumn")

    def test_method_short_descriptions(self):
        self.assertEqual(self.admin.contact_info.short_description, "Contact")
        self.assertEqual(
            self.admin.message_preview.short_description, "Message"
        )
        self.assertEqual(
            self.admin.message_stats.short_description, "Message Stats"
        )
        self.assertEqual(self.admin.contact_timing.short_description, "Timing")
        self.assertEqual(
            self.admin.priority_badge.short_description, "Priority"
        )
        self.assertEqual(
            self.admin.contact_analytics.short_description, "Contact Analytics"
        )
        self.assertEqual(
            self.admin.message_analytics.short_description, "Message Analytics"
        )
        self.assertEqual(
            self.admin.timing_info.short_description, "Timing Information"
        )


class TestContactAdminIntegration(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = User(username="testuser")

        self.site = AdminSite()
        self.admin = ContactAdmin(Contact, self.site)

    def test_fieldsets_structure(self):
        fieldsets = self.admin.fieldsets
        self.assertIsInstance(fieldsets, tuple)

        self.assertGreater(len(fieldsets), 1)

        basic_fieldset = fieldsets[0]
        self.assertEqual(basic_fieldset[0], "Contact Information")

        basic_fields = basic_fieldset[1]["fields"]
        self.assertIn("name", basic_fields)
        self.assertIn("email", basic_fields)

        message_fieldset = fieldsets[1]
        self.assertEqual(message_fieldset[0], "Message")
        message_fields = message_fieldset[1]["fields"]
        self.assertIn("message", message_fields)

    def test_list_filter_configuration(self):
        expected_filters = [
            RecentContactFilter,
            MessageLengthFilter,
            ("created_at", RangeDateTimeFilter),
            ("updated_at", RangeDateTimeFilter),
        ]
        self.assertEqual(self.admin.list_filter, expected_filters)

    def test_admin_registered(self):
        from django.contrib import admin

        self.assertIn(Contact, admin.site._registry)

        registered_admin = admin.site._registry[Contact]
        self.assertIsInstance(registered_admin, ContactAdmin)

    def test_unfold_integration(self):
        self.assertTrue(hasattr(self.admin, "compressed_fields"))
        self.assertTrue(hasattr(self.admin, "warn_unsaved_form"))


class TestContactAdminEdgeCases(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.user = User(username="testuser")

        self.site = AdminSite()
        self.admin = ContactAdmin(Contact, self.site)

    def test_contact_info_empty_email(self):
        empty_email_contact = Contact.objects.create(
            name="No Email User", email="", message="Test message"
        )

        result = self.admin.contact_info(empty_email_contact)

        self.assertIn("No Email User", result)
        self.assertIn("(no email)", result)

    def test_message_analytics_empty_message(self):
        empty_message_contact = Contact.objects.create(
            name="Empty Message User", email="empty@example.com", message=""
        )

        result = self.admin.message_analytics(empty_message_contact)

        self.assertIn("Reading Time", result)
        self.assertIn("0 seconds", result)

    def test_message_analytics_special_characters(self):
        special_contact = Contact.objects.create(
            name="Special User",
            email="special@example.com",
            message="Message with émojis 😊 and spëcial chàracters!",
        )

        result = self.admin.message_analytics(special_contact)

        self.assertIn("Reading Time", result)
        self.assertIn("Complexity", result)

    def test_timing_info_edge_hours(self):
        edge_time = datetime.datetime(
            2023, 12, 13, 9, 0, 0, tzinfo=ZoneInfo("UTC")
        )
        self.contact = Contact.objects.create(
            name="Edge User",
            email="edge@example.com",
            message="Edge case message",
        )
        self.contact.created_at = edge_time
        self.contact.save()

        result = self.admin.timing_info(self.contact)

        self.assertIn("09:00", result)
        self.assertIn("Yes", result)

    def test_contact_info_extremely_long_name(self):
        long_name = "A" * 90
        long_name_contact = Contact.objects.create(
            name=long_name, email="longname@example.com", message="Test message"
        )

        result = self.admin.contact_info(long_name_contact)

        self.assertIn("...", result)
        self.assertNotIn("A" * 90, result)
        self.assertIn("<div", result)
        self.assertIn("</div>", result)

    def test_message_preview_html_injection(self):
        html_contact = Contact.objects.create(
            name="HTML User",
            email="html@example.com",
            message="<script>alert('test')</script><b>Bold text</b>",
        )

        result = self.admin.message_preview(html_contact)

        self.assertNotIn("<script>", result)
        self.assertNotIn("<b>", result)
        self.assertIn("&lt;", result)

    def test_division_by_zero_protection(self):
        zero_contact = Contact.objects.create(
            name="",
            email="zero@example.com",
            message="",
        )

        try:
            self.admin.contact_info(zero_contact)
            self.admin.message_preview(zero_contact)
            self.admin.message_stats(zero_contact)
            self.admin.contact_timing(zero_contact)
            self.admin.priority_badge(zero_contact)
            self.admin.contact_analytics(zero_contact)
            self.admin.message_analytics(zero_contact)
            self.admin.timing_info(zero_contact)
        except ZeroDivisionError:
            self.fail(
                "Methods should handle zero values without ZeroDivisionError"
            )



import pytest
from asgiref.typing import (
    ASGIReceiveCallable,
    ASGISendCallable,
    HTTPResponseBodyEvent,
    HTTPResponseStartEvent,
    Scope,
)


@pytest.fixture
def asgi_app():
    async def fake_app(
        scope: Scope, receive: ASGIReceiveCallable, send: ASGISendCallable
    ):
        await send(
            HTTPResponseStartEvent(
                type="http.response.start",
                status=200,
                headers=[(b"content-type", b"text/plain")],
                trailers=False,
            )
        )
        await send(
            HTTPResponseBodyEvent(
                type="http.response.body", body=b"", more_body=False
            )
        )

    return fake_app


@pytest.fixture
def large_asgi_app():
    async def fake_app(
        scope: Scope, receive: ASGIReceiveCallable, send: ASGISendCallable
    ):
        await send(
            HTTPResponseStartEvent(
                type="http.response.start",
                status=200,
                headers=[
                    (b"content-length", b"10000"),
                    (b"content-type", b"text/plain"),
                ],
                trailers=False,
            )
        )
        await send(
            HTTPResponseBodyEvent(
                type="http.response.body", body=10000 * b"x", more_body=False
            )
        )

    return fake_app



from datetime import timedelta
from django.db.models import Count, Sum, Q, F, DecimalField
from django.db.models.functions import Coalesce
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from cart.models import Cart
from core.filters.camel_case_filters import CamelCaseTimeStampFilterSet
from core.filters.core import UUIDFilterMixin


class CartFilter(UUIDFilterMixin, CamelCaseTimeStampFilterSet):
    user = filters.NumberFilter(
        field_name="user__id",
        lookup_expr="exact",
        help_text=_("Filter by user ID"),
    )
    user__isnull = filters.BooleanFilter(
        field_name="user",
        lookup_expr="isnull",
        help_text=_("Filter carts with/without users"),
    )
    user_email = filters.CharFilter(
        field_name="user__email",
        lookup_expr="icontains",
        help_text=_("Filter by user email (partial match)"),
    )
    user_name = filters.CharFilter(
        method="filter_user_name",
        help_text=_("Filter by user full name (first or last name)"),
    )
    user__is_active = filters.BooleanFilter(
        field_name="user__is_active",
        help_text=_("Filter by active users"),
    )

    session_key = filters.CharFilter(
        field_name="session_key",
        lookup_expr="exact",
        help_text=_("Filter by exact session key"),
    )
    session_key__startswith = filters.CharFilter(
        field_name="session_key",
        lookup_expr="istartswith",
        help_text=_("Filter by session key prefix"),
    )
    has_session_key = filters.BooleanFilter(
        method="filter_has_session_key",
        help_text=_("Filter carts with/without session keys"),
    )

    is_guest = filters.BooleanFilter(
        method="filter_is_guest",
        help_text=_("Filter guest carts (True) or user carts (False)"),
    )
    cart_type = filters.ChoiceFilter(
        method="filter_cart_type",
        choices=[
            ("user", "User Cart"),
            ("guest", "Guest Cart"),
            ("anonymous", "Anonymous Cart"),
        ],
        help_text=_("Filter by cart type"),
    )

    last_activity = filters.DateTimeFilter(
        field_name="last_activity",
        help_text=_("Filter by exact last activity date"),
    )
    last_activity_after = filters.DateTimeFilter(
        field_name="last_activity",
        lookup_expr="gte",
        help_text=_("Filter carts with last activity after this date"),
    )
    last_activity_before = filters.DateTimeFilter(
        field_name="last_activity",
        lookup_expr="lte",
        help_text=_("Filter carts with last activity before this date"),
    )
    is_active = filters.BooleanFilter(
        method="filter_is_active",
        help_text=_(
            "Filter active/abandoned carts (based on 30-day inactivity)"
        ),
    )
    is_abandoned = filters.BooleanFilter(
        method="filter_is_abandoned",
        help_text=_("Filter abandoned carts (inactive for 30+ days)"),
    )
    days_inactive = filters.NumberFilter(
        method="filter_days_inactive",
        help_text=_("Filter carts inactive for at least X days"),
    )

    has_items = filters.BooleanFilter(
        method="filter_has_items",
        help_text=_("Filter carts that have/don't have items"),
    )
    min_items = filters.NumberFilter(
        method="filter_min_items",
        help_text=_("Filter carts with at least X total items (quantity)"),
    )
    max_items = filters.NumberFilter(
        method="filter_max_items",
        help_text=_("Filter carts with at most X total items (quantity)"),
    )
    min_unique_items = filters.NumberFilter(
        method="filter_min_unique_items",
        help_text=_("Filter carts with at least X unique items"),
    )
    max_unique_items = filters.NumberFilter(
        method="filter_max_unique_items",
        help_text=_("Filter carts with at most X unique items"),
    )

    min_total_value = filters.NumberFilter(
        method="filter_min_total_value",
        help_text=_("Filter carts with total value at least X"),
    )
    max_total_value = filters.NumberFilter(
        method="filter_max_total_value",
        help_text=_("Filter carts with total value at most X"),
    )
    has_discounts = filters.BooleanFilter(
        method="filter_has_discounts",
        help_text=_("Filter carts with/without discounted items"),
    )

    class Meta:
        model = Cart
        fields = {
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "uuid": ["exact"],
            "id": ["exact", "in"],
            "user": ["exact", "isnull"],
            "session_key": ["exact", "icontains", "istartswith"],
            "last_activity": ["exact", "gte", "lte", "date"],
        }

    def filter_user_name(self, queryset, name, value):
        """Filter by user's first or last name."""
        if value:
            return queryset.filter(
                Q(user__first_name__icontains=value)
                | Q(user__last_name__icontains=value)
            )
        return queryset

    def filter_has_session_key(self, queryset, name, value):
        """Filter carts based on session key presence."""
        if value is True:
            return queryset.exclude(
                Q(session_key__isnull=True) | Q(session_key__exact="")
            )
        elif value is False:
            return queryset.filter(
                Q(session_key__isnull=True) | Q(session_key__exact="")
            )
        return queryset

    def filter_is_guest(self, queryset, name, value):
        """Filter guest (no user) vs user carts."""
        if value is True:
            return queryset.filter(user__isnull=True)
        elif value is False:
            return queryset.filter(user__isnull=False)
        return queryset

    def filter_cart_type(self, queryset, name, value):
        """Filter by specific cart type."""
        if value == "user":
            return queryset.filter(user__isnull=False)
        elif value == "guest":
            return queryset.filter(user__isnull=True).exclude(
                Q(session_key__isnull=True) | Q(session_key__exact="")
            )
        elif value == "anonymous":
            return queryset.filter(user__isnull=True).filter(
                Q(session_key__isnull=True) | Q(session_key__exact="")
            )
        return queryset

    def filter_is_active(self, queryset, name, value):
        """Filter active carts (activity within 30 days)."""
        cutoff = timezone.now() - timedelta(days=30)
        if value is True:
            return queryset.filter(last_activity__gte=cutoff)
        elif value is False:
            return queryset.filter(last_activity__lt=cutoff)
        return queryset

    def filter_is_abandoned(self, queryset, name, value):
        """Filter abandoned carts (inactive 30+ days)."""
        cutoff = timezone.now() - timedelta(days=30)
        if value is True:
            return queryset.filter(last_activity__lt=cutoff)
        elif value is False:
            return queryset.filter(last_activity__gte=cutoff)
        return queryset

    def filter_days_inactive(self, queryset, name, value):
        """Filter carts inactive for at least X days."""
        if value is not None:
            cutoff = timezone.now() - timedelta(days=value)
            return queryset.filter(last_activity__lte=cutoff)
        return queryset

    def filter_has_items(self, queryset, name, value):
        """Filter carts based on item presence."""
        if value is True:
            return queryset.annotate(item_count=Count("items")).filter(
                item_count__gt=0
            )
        elif value is False:
            return queryset.annotate(item_count=Count("items")).filter(
                item_count=0
            )
        return queryset

    def filter_min_items(self, queryset, name, value):
        """Filter carts with minimum total quantity."""
        if value is not None:
            return queryset.annotate(
                total_quantity=Coalesce(Sum("items__quantity"), 0)
            ).filter(total_quantity__gte=value)
        return queryset

    def filter_max_items(self, queryset, name, value):
        """Filter carts with maximum total quantity."""
        if value is not None:
            return queryset.annotate(
                total_quantity=Coalesce(Sum("items__quantity"), 0)
            ).filter(total_quantity__lte=value)
        return queryset

    def filter_min_unique_items(self, queryset, name, value):
        """Filter carts with minimum unique items."""
        if value is not None:
            return queryset.annotate(
                unique_items=Count("items", distinct=True)
            ).filter(unique_items__gte=value)
        return queryset

    def filter_max_unique_items(self, queryset, name, value):
        """Filter carts with maximum unique items."""
        if value is not None:
            return queryset.annotate(
                unique_items=Count("items", distinct=True)
            ).filter(unique_items__lte=value)
        return queryset

    def filter_min_total_value(self, queryset, name, value):
        """Filter carts with minimum total value."""
        if value is not None:
            return queryset.annotate(
                total_value=Coalesce(
                    Sum(
                        F("items__quantity") * F("items__product__price"),
                        output_field=DecimalField(),
                    ),
                    0,
                    output_field=DecimalField(),
                )
            ).filter(total_value__gte=value)
        return queryset

    def filter_max_total_value(self, queryset, name, value):
        """Filter carts with maximum total value."""
        if value is not None:
            return queryset.annotate(
                total_value=Coalesce(
                    Sum(
                        F("items__quantity") * F("items__product__price"),
                        output_field=DecimalField(),
                    ),
                    0,
                    output_field=DecimalField(),
                )
            ).filter(total_value__lte=value)
        return queryset

    def filter_has_discounts(self, queryset, name, value):
        """Filter carts with discounted items."""
        if value is True:
            return queryset.filter(items__discount_value__gt=0).distinct()
        elif value is False:
            return queryset.exclude(items__discount_value__gt=0).distinct()
        return queryset



import logging
from datetime import timedelta

from celery import shared_task
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from order.enum.status import OrderStatus
from order.models import Order, OrderHistory
from order.services import OrderService
from order.shipping import ShippingService

logger = logging.getLogger(__name__)


@shared_task(bind=True, max_retries=3, default_retry_delay=300)
def send_order_confirmation_email(self, order_id: int) -> bool:
    try:
        order = Order.objects.get(id=order_id)
        context = {
            "order": order,
            "items": order.items.all(),
            "site_name": getattr(settings, "SITE_NAME", "Our Shop"),
            "info_email": getattr(
                settings, "INFO_EMAIL", "support@example.com"
            ),
            "site_url": getattr(settings, "SITE_URL", "https://example.com"),
        }

        subject = _("Order Confirmation - #{order_id}").format(
            order_id=order.id
        )
        text_content = render_to_string(
            "emails/order_confirmation.txt", context
        )
        html_content = render_to_string(
            "emails/order_confirmation.html", context
        )

        msg = EmailMultiAlternatives(
            subject,
            text_content,
            getattr(settings, "DEFAULT_FROM_EMAIL", "noreply@example.com"),
            [order.email],
            reply_to=[getattr(settings, "INFO_EMAIL", "support@example.com")],
        )
        msg.attach_alternative(html_content, "text/html")

        msg.send()

        logger.info(
            f"Order confirmation email sent for order #{order.id}",
            extra={"order_id": order.id, "email": order.email},
        )

        OrderHistory.log_note(
            order=order,
            note=f"Order confirmation email sent to {order.email}",
        )

        return True

    except Order.DoesNotExist:
        logger.error(
            f"Could not send confirmation email - Order #{order_id} not found",
            extra={"order_id": order_id},
        )
        return False

    except Exception as e:
        logger.error(
            f"Error sending order confirmation email for order #{order_id}: {e!s}",
            extra={"order_id": order_id, "error": str(e)},
        )

        retry_count = self.request.retries
        max_retries = self.max_retries

        if retry_count < max_retries:
            logger.info(
                f"Retrying send_order_confirmation_email for order #{order_id} "
                f"(attempt {retry_count + 1}/{max_retries + 1})"
            )
            raise self.retry(exc=e) from e

        return False


@shared_task(bind=True, max_retries=3, default_retry_delay=300)
def send_order_status_update_email(
    self, order_id: int, status: OrderStatus
) -> bool:
    try:
        order = Order.objects.get(id=order_id)

        if status in [OrderStatus.PENDING]:
            return True

        context = {
            "order": order,
            "status": status,
            "site_name": getattr(settings, "SITE_NAME", "Our Shop"),
            "info_email": getattr(
                settings, "INFO_EMAIL", "support@example.com"
            ),
            "site_url": getattr(settings, "SITE_URL", "https://example.com"),
        }

        template_base = f"emails/order_{status.lower()}"

        subject = _("Order #{order_id} Status Update - {status}").format(
            order_id=order.id, status=OrderStatus(status).label
        )

        try:
            text_content = render_to_string(f"{template_base}.txt", context)
            html_content = render_to_string(f"{template_base}.html", context)
        except Exception:
            logger.warning(
                f"Template {template_base} not found, using generic template",
                extra={"order_id": order_id, "status": status},
            )
            text_content = render_to_string(
                "emails/order_status_generic.txt", context
            )
            html_content = render_to_string(
                "emails/order_status_generic.html", context
            )

        msg = EmailMultiAlternatives(
            subject,
            text_content,
            getattr(settings, "DEFAULT_FROM_EMAIL", "noreply@example.com"),
            [order.email],
            reply_to=[getattr(settings, "INFO_EMAIL", "support@example.com")],
        )
        msg.attach_alternative(html_content, "text/html")

        msg.send()

        logger.info(
            f"Order status update email sent for order #{order.id} - Status: {status}",
            extra={
                "order_id": order.id,
                "status": status,
                "email": order.email,
            },
        )

        OrderHistory.log_note(
            order=order,
            note=f"Status update email sent to {order.email} for {status} status",
        )

        return True

    except Order.DoesNotExist:
        logger.error(
            f"Could not send status update email - Order #{order_id} not found",
            extra={"order_id": order_id, "status": status},
        )
        return False

    except Exception as e:
        logger.error(
            f"Error sending order status update email for order #{order_id}: {e!s}",
            extra={"order_id": order_id, "status": status, "error": str(e)},
        )

        if self.request.retries < self.max_retries:
            logger.info(
                f"Retrying send_order_status_update_email for order #{order_id} "
                f"(attempt {self.request.retries + 1}/{self.max_retries + 1})"
            )
            raise self.retry(exc=e) from e

        return False


@shared_task(bind=True, max_retries=3, default_retry_delay=300)
def send_shipping_notification_email(self, order_id: int) -> bool:
    try:
        order = Order.objects.get(id=order_id)

        if not order.tracking_number or not order.shipping_carrier:
            logger.warning(
                f"Attempted to send shipping notification for order #{order_id} without tracking info",
                extra={"order_id": order_id},
            )
            return False

        context = {
            "order": order,
            "tracking_number": order.tracking_number,
            "carrier": order.shipping_carrier,
            "site_name": getattr(settings, "SITE_NAME", "Our Shop"),
            "info_email": getattr(
                settings, "INFO_EMAIL", "support@example.com"
            ),
            "site_url": getattr(settings, "SITE_URL", "https://example.com"),
        }

        subject = _("Your Order #{order_id} Has Shipped").format(
            order_id=order.id
        )
        text_content = render_to_string("emails/order_shipped.txt", context)
        html_content = render_to_string("emails/order_shipped.html", context)

        msg = EmailMultiAlternatives(
            subject,
            text_content,
            getattr(settings, "DEFAULT_FROM_EMAIL", "noreply@example.com"),
            [order.email],
            reply_to=[getattr(settings, "INFO_EMAIL", "support@example.com")],
        )
        msg.attach_alternative(html_content, "text/html")

        msg.send()

        logger.info(
            f"Shipping confirmation email sent for order #{order.id}",
            extra={
                "order_id": order.id,
                "tracking_number": order.tracking_number,
                "carrier": order.shipping_carrier,
            },
        )

        OrderHistory.log_note(
            order=order,
            note=f"Shipping confirmation email sent to {order.email} with "
            f"tracking number {order.tracking_number} via {order.shipping_carrier}",
        )

        return True

    except Order.DoesNotExist:
        logger.error(
            f"Could not send shipping confirmation email - Order #{order_id} not found",
            extra={"order_id": order_id},
        )
        return False

    except Exception as e:
        logger.error(
            f"Error sending shipping confirmation email for order #{order_id}: {e!s}",
            extra={"order_id": order_id, "error": str(e)},
        )

        if self.request.retries < self.max_retries:
            raise self.retry(exc=e) from e

        return False


@shared_task
def generate_order_invoice(order_id: int) -> bool:
    try:
        order = Order.objects.get(id=order_id)

        # @TODO - In a real implementation, you would generate a PDF using a library
        # like weasyprint or reportlab, and store it in a file field on the Order model
        # or in a separate Invoice model

        # For this example, we'll just log the action
        logger.info(
            f"Invoice generated for order #{order.id}",
            extra={"order_id": order.id},
        )

        OrderHistory.log_note(
            order=order,
            note=f"Invoice generated for order #{order.id}",
        )

        return True

    except Order.DoesNotExist:
        logger.error(
            f"Could not generate invoice - Order #{order_id} not found",
            extra={"order_id": order_id},
        )
        return False

    except Exception as e:
        logger.error(
            f"Error generating invoice for order #{order_id}: {e!s}",
            extra={"order_id": order_id, "error": str(e)},
        )
        return False


@shared_task
def check_pending_orders() -> int:
    try:
        one_day_ago = timezone.now() - timedelta(days=1)
        pending_orders = Order.objects.filter(
            status=OrderStatus.PENDING, created_at__lt=one_day_ago
        )

        count = 0
        for order in pending_orders:
            context = {
                "order": order,
                "site_name": settings.SITE_NAME,
                "info_email": settings.INFO_EMAIL,
                "site_url": settings.SITE_URL,
            }

            subject = _("Reminder: Complete Your Order #{order_id}").format(
                order_id=order.id
            )
            text_content = render_to_string(
                "emails/order_pending_reminder.txt", context
            )
            html_content = render_to_string(
                "emails/order_pending_reminder.html", context
            )

            msg = EmailMultiAlternatives(
                subject,
                text_content,
                settings.DEFAULT_FROM_EMAIL,
                [order.email],
                reply_to=[settings.INFO_EMAIL],
            )
            msg.attach_alternative(html_content, "text/html")

            msg.send()

            logger.info(
                f"Pending order reminder sent for order #{order.id}",
                extra={"order_id": order.id, "email": order.email},
            )

            OrderHistory.log_note(
                order=order,
                note=f"Pending order reminder email sent to {order.email}",
            )

            count += 1

        return count

    except Exception as e:
        logger.error(
            f"Error checking pending orders: {e!s}", extra={"error": str(e)}
        )
        return 0


@shared_task
def update_order_statuses_from_shipping() -> int:
    try:
        shipped_orders = Order.objects.filter(
            status=OrderStatus.SHIPPED, tracking_number__isnull=False
        ).exclude(tracking_number="")

        count = 0

        for order in shipped_orders:
            if not order.shipping_carrier:
                continue

            try:
                tracking_info = ShippingService.get_tracking_info(
                    order.tracking_number, order.shipping_carrier
                )

                if tracking_info.get("status") == OrderStatus.DELIVERED:
                    OrderService.update_order_status(
                        order, OrderStatus.DELIVERED
                    )

                    send_order_status_update_email.delay(
                        order.id, OrderStatus.DELIVERED
                    )

                    count += 1

            except Exception as inner_e:
                logger.error(
                    f"Error updating shipping status for order #{order.id}: {inner_e!s}",
                    extra={"order_id": order.id, "error": str(inner_e)},
                )

        return count

    except Exception as e:
        logger.error(
            f"Error updating order statuses from shipping: {e!s}",
            extra={"error": str(e)},
        )
        return 0



import re
from djangorestframework_camel_case.util import (
    camel_to_underscore,
    camelize_re,
    underscore_to_camel as underscore_to_camel_callback,
)
from django_filters import rest_framework as filters
from django.utils.translation import gettext_lazy as _
from drf_spectacular.extensions import OpenApiFilterExtension
from drf_spectacular.plumbing import build_parameter_type
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import OpenApiParameter


def snake_to_camel(snake_str):
    return re.sub(camelize_re, underscore_to_camel_callback, snake_str)


class CamelCaseFilterMixin:
    def __init__(self, data=None, *args, **kwargs):
        if data is not None:
            data = self._convert_data_to_snake_case(data)

        super().__init__(data, *args, **kwargs)

        self._camel_case_fields = {}

        if hasattr(self, "base_filters"):
            new_filters = {}
            for snake_name, filter_instance in self.base_filters.items():
                camel_name = snake_to_camel(snake_name)

                if camel_name != snake_name:
                    new_filters[camel_name] = filter_instance
                    self._camel_case_fields[camel_name] = snake_name
                else:
                    new_filters[snake_name] = filter_instance

            self.base_filters = new_filters

    def _convert_data_to_snake_case(self, data):
        if not data:
            return data

        if hasattr(data, "getlist"):
            from django.http import QueryDict

            new_data = QueryDict(mutable=True)
            for key in data:
                snake_key = camel_to_underscore(key)
                values = data.getlist(key)
                for value in values:
                    new_data.appendlist(snake_key, value)
            return new_data

        new_data = {}
        for key, value in data.items():
            snake_key = camel_to_underscore(key)
            new_data[snake_key] = value
        return new_data


class CamelCaseTimeStampFilterSet(CamelCaseFilterMixin, filters.FilterSet):
    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text="Filter items created after this date",
    )
    created_before = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text="Filter items created before this date",
    )
    updated_after = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="gte",
        help_text="Filter items updated after this date",
    )
    updated_before = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="lte",
        help_text="Filter items updated before this date",
    )

    class Meta:
        abstract = True


class CamelCasePublishableTimeStampFilterSet(
    CamelCaseFilterMixin, filters.FilterSet
):
    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text=_("Filter items created after this date"),
    )
    created_before = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text=_("Filter items created before this date"),
    )
    updated_after = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="gte",
        help_text=_("Filter items updated after this date"),
    )
    updated_before = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="lte",
        help_text=_("Filter items updated before this date"),
    )

    is_published = filters.BooleanFilter(
        field_name="is_published",
        help_text=_("Filter by published status"),
    )
    published_after = filters.DateTimeFilter(
        field_name="published_at",
        lookup_expr="gte",
        help_text=_("Filter items published after this date"),
    )
    published_before = filters.DateTimeFilter(
        field_name="published_at",
        lookup_expr="lte",
        help_text=_("Filter items published before this date"),
    )
    currently_published = filters.BooleanFilter(
        method="filter_currently_published",
        help_text=_(
            "Filter items that are currently published (published_at <= now and is_published=True)"
        ),
    )

    def filter_currently_published(self, queryset, name, value):
        """Filter items that are currently published."""
        if value:
            from django.utils import timezone
            from django.db.models import Q

            today = timezone.now()
            return queryset.filter(
                Q(published_at__lte=today, is_published=True)
                | Q(published_at__isnull=True, is_published=True)
            )
        return queryset

    class Meta:
        abstract = True
        fields = {
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "published_at": ["gte", "lte", "date"],
            "is_published": ["exact"],
        }


class CamelCaseFilterExtension(OpenApiFilterExtension):
    target_class = "core.filters.camel_case_filters.CamelCaseFilterMixin"
    priority = 1

    def get_schema_operation_parameters(self, auto_schema, *args, **kwargs):
        parameters = []

        for camel_name, filter_field in self.target.base_filters.items():
            getattr(filter_field, "field_name", camel_name)
            help_text = getattr(filter_field, "help_text", "")

            if isinstance(filter_field, filters.DateTimeFilter):
                schema_type = OpenApiTypes.DATETIME
            elif isinstance(filter_field, filters.BooleanFilter):
                schema_type = OpenApiTypes.BOOL
            elif isinstance(filter_field, filters.NumberFilter):
                schema_type = OpenApiTypes.NUMBER
            elif isinstance(filter_field, filters.UUIDFilter):
                schema_type = OpenApiTypes.UUID
            else:
                schema_type = OpenApiTypes.STR

            parameters.append(
                build_parameter_type(
                    name=camel_name,
                    schema=schema_type,
                    location=OpenApiParameter.QUERY,
                    description=help_text,
                )
            )

        return parameters



# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contact', '0002_alter_contact_email_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='contact',
            name='contact_con_email_2ebc86_idx',
        ),
        migrations.AddIndex(
            model_name='contact',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['email'], name='contact_email_ix'),
        ),
    ]



from django.contrib.postgres.operations import BtreeGinExtension
from django.contrib.postgres.operations import TrigramExtension
from django.db import migrations


class Migration(migrations.Migration):
    run_before = [
        ("blog", "0001_initial"),
        ("cart", "0001_initial"),
        ("country", "0001_initial"),
        ("notification", "0001_initial"),
        ("order", "0001_initial"),
        ("pay_way", "0001_initial"),
        ("product", "0001_initial"),
        ("region", "0001_initial"),
        ("user", "0001_initial"),
        ("vat", "0001_initial"),
    ]
    initial = True

    operations = [
        TrigramExtension(),
        BtreeGinExtension(),
    ]



from datetime import timedelta
from django.utils import timezone
from django.urls import reverse
from django.test import TransactionTestCase
from rest_framework.test import APIClient
import pytest

from blog.factories.author import BlogAuthorFactory
from blog.factories.post import BlogPostFactory
from blog.models.author import BlogAuthor
from user.factories.account import UserAccountFactory
from user.models.account import UserAccount


@pytest.mark.django_db(transaction=True)
class BlogAuthorFilterTest(TransactionTestCase):
    def setUp(self):
        BlogAuthor.objects.all().delete()
        UserAccount.objects.all().delete()

        self.client = APIClient()

        self.now = timezone.now()

        self.user1 = UserAccountFactory(
            first_name="John", last_name="Doe", email="john.doe@example.com"
        )
        self.user2 = UserAccountFactory(
            first_name="Jane", last_name="Smith", email="jane.smith@example.com"
        )
        self.user3 = UserAccountFactory(
            first_name="Bob",
            last_name="Johnson",
            email="bob.johnson@example.com",
        )
        self.user4 = UserAccountFactory(
            first_name="Alice",
            last_name="Williams",
            email="alice.williams@example.com",
        )

        self.like_users = []
        for i in range(20):
            user = UserAccountFactory(
                first_name=f"LikeUser{i}",
                last_name=f"Test{i}",
                email=f"likeuser{i}@test.com",
            )
            self.like_users.append(user)

        self.author1 = BlogAuthorFactory(
            user=self.user1, website="https://johndoe.com"
        )
        self.author1.created_at = self.now - timedelta(days=90)
        self.author1.save()
        self.author1.set_current_language("en")
        self.author1.bio = "Senior tech writer with 10 years experience"
        self.author1.save()

        self.author2 = BlogAuthorFactory(
            user=self.user2,
            website="",
        )
        self.author2.created_at = self.now - timedelta(days=30)
        self.author2.save()
        self.author2.set_current_language("en")
        self.author2.bio = "Freelance blogger specializing in travel"
        self.author2.save()

        self.author3 = BlogAuthorFactory(
            user=self.user3, website="https://bobjohnson.io"
        )
        self.author3.created_at = self.now - timedelta(days=7)
        self.author3.save()
        self.author3.set_current_language("en")
        self.author3.bio = "Tech enthusiast and software developer"
        self.author3.save()

        self.author4 = BlogAuthorFactory(
            user=self.user4,
            website="",
        )
        self.author4.created_at = self.now - timedelta(hours=1)
        self.author4.save()
        self.author4.set_current_language("en")
        self.author4.bio = "New to blogging"
        self.author4.save()

        self.author1_posts = []
        for i in range(5):
            post = BlogPostFactory(author=self.author1, is_published=True)
            for j in range(10):
                user_index = (i * 10 + j) % len(self.like_users)
                post.likes.add(self.like_users[user_index])
            self.author1_posts.append(post)

        self.author2_posts = []
        for i in range(3):
            post = BlogPostFactory(author=self.author2, is_published=True)
            for j in range(5):
                user_index = (i * 5 + j) % len(self.like_users)
                post.likes.add(self.like_users[user_index])
            self.author2_posts.append(post)

        self.author3_post = BlogPostFactory(
            author=self.author3, is_published=True
        )

    def test_timestamp_filters(self):
        url = reverse("blog-author-list")

        created_after = self.now - timedelta(days=60)
        response = self.client.get(
            url, {"created_after": created_after.isoformat()}
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        expected_authors = [self.author2.id, self.author3.id, self.author4.id]

        for author_id in expected_authors:
            self.assertIn(author_id, result_ids)

        created_before = self.now - timedelta(days=14)
        response = self.client.get(
            url, {"created_before": created_before.isoformat()}
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        expected_authors = [self.author1.id, self.author2.id]

        for author_id in expected_authors:
            self.assertIn(author_id, result_ids)

    def test_uuid_filter(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"uuid": str(self.author2.uuid)})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author2.id, result_ids)
        author_found = any(
            r["id"] == self.author2.id for r in response.data["results"]
        )
        self.assertTrue(author_found)

    def test_user_filters(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"user": self.user1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author1.id, result_ids)

        response = self.client.get(url, {"user_email": "jane"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author2.id, result_ids)

        response = self.client.get(url, {"first_name": "ob"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author3.id, result_ids)

        response = self.client.get(url, {"last_name": "williams"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author4.id, result_ids)

    def test_full_name_filter(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"full_name": "John"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author1.id, result_ids)
        self.assertIn(self.author3.id, result_ids)

        response = self.client.get(url, {"full_name": "Jane Smith"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author2.id, result_ids)

    def test_website_filters(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"has_website": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        authors_with_website = [self.author1.id, self.author3.id]
        for author_id in authors_with_website:
            self.assertIn(author_id, result_ids)

        response = self.client.get(url, {"has_website": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        authors_without_website = [self.author2.id, self.author4.id]
        for author_id in authors_without_website:
            self.assertIn(author_id, result_ids)

        response = self.client.get(url, {"website": "johnson"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author3.id, result_ids)

    def test_bio_filter(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"bio": "tech"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        expected_authors = [self.author1.id, self.author3.id]
        for author_id in expected_authors:
            self.assertIn(author_id, result_ids)

    def test_post_count_filters(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"min_posts": 3})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertIn(self.author1.id, result_ids)
        self.assertIn(self.author2.id, result_ids)

        response = self.client.get(url, {"max_posts": 1})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertIn(self.author3.id, result_ids)
        self.assertIn(self.author4.id, result_ids)

        response = self.client.get(url, {"has_posts": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        authors_with_posts = [self.author1.id, self.author2.id, self.author3.id]
        for author_id in authors_with_posts:
            self.assertIn(author_id, result_ids)

        response = self.client.get(url, {"has_posts": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertIn(self.author4.id, result_ids)

    def test_like_count_filters(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"min_total_likes": 20})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertIn(self.author1.id, result_ids)

        response = self.client.get(url, {"has_likes": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        authors_with_likes = [self.author1.id, self.author2.id]
        for author_id in authors_with_likes:
            self.assertIn(author_id, result_ids)

        response = self.client.get(url, {"has_likes": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        authors_without_likes = [self.author3.id, self.author4.id]
        for author_id in authors_without_likes:
            self.assertIn(author_id, result_ids)

    def test_special_filters(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"most_active": "true"})
        self.assertEqual(response.status_code, 200)
        results = response.data["results"]

        expected_authors_with_posts = [
            self.author1.id,
            self.author2.id,
            self.author3.id,
        ]
        result_ids = [r["id"] for r in results]

        for author_id in expected_authors_with_posts:
            self.assertIn(
                author_id,
                result_ids,
                f"Author {author_id} should be in results",
            )

        self.assertIn(self.author1.id, result_ids)
        self.assertIn(self.author2.id, result_ids)
        self.assertIn(self.author3.id, result_ids)

        response = self.client.get(url, {"most_liked": "true"})
        self.assertEqual(response.status_code, 200)
        results = response.data["results"]

        expected_authors_with_likes = [self.author1.id, self.author2.id]
        result_ids = [r["id"] for r in results]

        for author_id in expected_authors_with_likes:
            self.assertIn(
                author_id,
                result_ids,
                f"Author {author_id} should be in results",
            )

    def test_camel_case_filters(self):
        url = reverse("blog-author-list")

        created_after = self.now - timedelta(days=60)
        response = self.client.get(
            url,
            {
                "createdAfter": created_after.isoformat(),
                "hasWebsite": "true",
                "minPosts": 2,
            },
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertGreaterEqual(len(result_ids), 0)

        self.assertEqual(response.status_code, 200)

        response = self.client.get(url, {"userEmail": "alice"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.author4.id, result_ids)

    def test_complex_filter_combinations(self):
        url = reverse("blog-author-list")

        created_after = self.now - timedelta(days=35)
        response = self.client.get(
            url,
            {
                "createdAfter": created_after.isoformat(),
                "hasWebsite": "true",
                "hasPosts": "true",
                "bio": "tech",
            },
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertIn(self.author3.id, result_ids)

        response = self.client.get(
            url, {"minPosts": 2, "minTotalLikes": 10, "hasWebsite": "true"}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertIn(self.author1.id, result_ids)

    def test_filter_with_ordering(self):
        url = reverse("blog-author-list")

        response = self.client.get(url, {"ordering": "-createdAt"})
        self.assertEqual(response.status_code, 200)

        results = response.data["results"]

        test_author_results = []
        for result in results:
            if result["id"] in [
                self.author1.id,
                self.author2.id,
                self.author3.id,
                self.author4.id,
            ]:
                test_author_results.append(result)

        self.assertEqual(len(test_author_results), 4)

        self.assertEqual(test_author_results[0]["id"], self.author4.id)
        self.assertEqual(test_author_results[1]["id"], self.author3.id)
        self.assertEqual(test_author_results[2]["id"], self.author2.id)
        self.assertEqual(test_author_results[3]["id"], self.author1.id)

    def tearDown(self):
        BlogAuthor.objects.all().delete()
        UserAccount.objects.all().delete()



from __future__ import annotations

from django.conf import settings
from drf_spectacular.utils import extend_schema_view


from core.api.views import BaseModelViewSet
from core.utils.serializers import (
    create_schema_view_config,
    RequestSerializersConfig,
    ResponseSerializersConfig,
)
from core.utils.views import cache_methods
from product.filters.category import ProductCategoryFilter
from product.models.category import ProductCategory
from product.serializers.category import (
    ProductCategoryDetailSerializer,
    ProductCategorySerializer,
    ProductCategoryWriteSerializer,
)

req_serializers: RequestSerializersConfig = {
    "create": ProductCategoryWriteSerializer,
    "update": ProductCategoryWriteSerializer,
    "partial_update": ProductCategoryWriteSerializer,
}

res_serializers: ResponseSerializersConfig = {
    "create": ProductCategoryDetailSerializer,
    "list": ProductCategorySerializer,
    "retrieve": ProductCategoryDetailSerializer,
    "update": ProductCategoryDetailSerializer,
    "partial_update": ProductCategoryDetailSerializer,
}


@extend_schema_view(
    **create_schema_view_config(
        model_class=ProductCategory,
        display_config={
            "tag": "Product Categories",
        },
        request_serializers=req_serializers,
        response_serializers=res_serializers,
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class ProductCategoryViewSet(BaseModelViewSet):
    queryset = ProductCategory.objects.all()
    filterset_class = ProductCategoryFilter
    response_serializers = res_serializers
    request_serializers = req_serializers
    ordering_fields = [
        "id",
        "sort_order",
        "level",
        "lft",
        "rght",
        "tree_id",
        "created_at",
        "updated_at",
    ]
    ordering = ["tree_id", "lft"]
    search_fields = ["translations__name", "translations__description", "slug"]



from rest_framework import serializers


class TestURLFixerMixin:
    """
    Test mixin that temporarily modifies serializer field output methods to use relative URLs.
    This fixes the discrepancy between serializer output in tests (with http://testserver/)
    and the expected output (relative URLs).
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._orig_url_field_to_representation = (
            serializers.HyperlinkedRelatedField.to_representation
        )
        cls._orig_image_field_to_representation = (
            serializers.ImageField.to_representation
        )

        def patched_url_field_to_representation(self, value):
            url = cls._orig_url_field_to_representation(self, value)
            if url and url.startswith("http://testserver"):
                return url[len("http://testserver") :]
            return url

        def patched_image_field_to_representation(self, value):
            url = cls._orig_image_field_to_representation(self, value)
            if url and url.startswith("http://testserver"):
                return url[len("http://testserver") :]
            return url

        serializers.HyperlinkedRelatedField.to_representation = (
            patched_url_field_to_representation
        )
        serializers.ImageField.to_representation = (
            patched_image_field_to_representation
        )

    @classmethod
    def tearDownClass(cls):
        serializers.HyperlinkedRelatedField.to_representation = (
            cls._orig_url_field_to_representation
        )
        serializers.ImageField.to_representation = (
            cls._orig_image_field_to_representation
        )
        super().tearDownClass()



import json
from unittest.mock import Mock

import pytest
from django.db import models
from django.test import TestCase
from rest_framework import serializers
from rest_framework.test import APIRequestFactory
from rest_framework.viewsets import ModelViewSet
from core.utils.serializers import (
    TranslatedFieldExtended,
    create_schema_view_config,
    RequestSerializersConfig,
    ResponseSerializersConfig,
)


class MockSerializerModel(models.Model):
    field1 = models.IntegerField()
    field2 = models.CharField(max_length=100)
    field3 = models.IntegerField()
    field4 = models.IntegerField()

    class Meta:
        app_label = "test_app"
        verbose_name = "Mock Serializer Model"
        verbose_name_plural = "Mock Serializer Models"

    def __str__(self):
        return self.field2


class DummySerializer(serializers.Serializer):
    field1 = serializers.CharField()
    field2 = serializers.IntegerField()


class AnotherDummySerializer(serializers.Serializer):
    field3 = serializers.CharField()
    field4 = serializers.BooleanField()


class DummyTestSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField(max_length=100)
    description = serializers.CharField(required=False)


class DummyTestDetailSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField(max_length=100)
    description = serializers.CharField()
    created_at = serializers.DateTimeField()


class DummyTestWriteSerializer(serializers.Serializer):
    name = serializers.CharField(max_length=100)
    description = serializers.CharField(required=False)


class DummyTestViewSet(ModelViewSet):
    response_serializers = {
        "create": DummyTestDetailSerializer,
        "update": DummyTestDetailSerializer,
        "partial_update": DummyTestDetailSerializer,
    }
    request_serializers = {
        "create": DummyTestWriteSerializer,
        "update": DummyTestWriteSerializer,
    }


class DummyTestViewSetWithSerializerClass(ModelViewSet):
    serializer_class = DummyTestSerializer


class DummyTestViewSetMissingSerializers(ModelViewSet):
    pass


@pytest.fixture
def request_factory():
    return APIRequestFactory()


@pytest.fixture
def mock_request():
    request = Mock()
    request.user = Mock()
    request.query_params = {}
    return request


class TestTranslatedFieldExtended(TestCase):
    def test_to_internal_value_with_valid_data(self):
        data = {
            "en": {"field1": "value1", "field2": 123},
            "fr": {"field1": "valeur1", "field2": 456},
        }
        field = TranslatedFieldExtended(serializer_class=DummySerializer)
        result = field.to_internal_value(json.dumps(data))
        expected_result = {
            "en": {"field1": "value1", "field2": 123},
            "fr": {"field1": "valeur1", "field2": 456},
        }
        self.assertEqual(result, expected_result)


class TestCreateSchemaViewConfig(TestCase):
    def test_basic_config_generation(self):
        req_serializers: RequestSerializersConfig = {
            "create": DummyTestWriteSerializer,
            "update": DummyTestWriteSerializer,
            "partial_update": DummyTestWriteSerializer,
        }

        res_serializers: ResponseSerializersConfig = {
            "create": DummyTestDetailSerializer,
            "list": DummyTestSerializer,
            "retrieve": DummyTestDetailSerializer,
            "update": DummyTestDetailSerializer,
            "partial_update": DummyTestDetailSerializer,
        }

        config = create_schema_view_config(
            model_class=MockSerializerModel,
            display_config={
                "tag": "Test Models",
            },
            request_serializers=req_serializers,
            response_serializers=res_serializers,
        )

        assert "list" in config
        assert "create" in config
        assert "retrieve" in config
        assert "update" in config
        assert "partial_update" in config
        assert "destroy" in config

        for _operation_name, decorator in config.items():
            assert callable(decorator)
            assert hasattr(decorator, "__name__")

    def test_config_with_error_serializer(self):
        req_serializers: RequestSerializersConfig = {
            "create": DummyTestWriteSerializer,
            "update": DummyTestWriteSerializer,
            "partial_update": DummyTestWriteSerializer,
        }

        res_serializers: ResponseSerializersConfig = {
            "create": DummyTestDetailSerializer,
            "list": DummyTestSerializer,
            "retrieve": DummyTestDetailSerializer,
            "update": DummyTestDetailSerializer,
            "partial_update": DummyTestDetailSerializer,
        }

        config = create_schema_view_config(
            model_class=MockSerializerModel,
            display_config={
                "tag": "Test Models",
            },
            request_serializers=req_serializers,
            response_serializers=res_serializers,
            error_serializer=DummyTestSerializer,
        )

        expected_operations = [
            "list",
            "create",
            "retrieve",
            "update",
            "partial_update",
            "destroy",
        ]
        for operation in expected_operations:
            assert operation in config
            assert callable(config[operation])

    def test_config_with_additional_responses(self):
        additional_responses = {
            "list": {422: DummyTestSerializer},
            "create": {409: DummyTestSerializer},
        }

        req_serializers: RequestSerializersConfig = {
            "create": DummyTestWriteSerializer,
            "update": DummyTestWriteSerializer,
            "partial_update": DummyTestWriteSerializer,
        }

        res_serializers: ResponseSerializersConfig = {
            "create": DummyTestDetailSerializer,
            "list": DummyTestSerializer,
            "retrieve": DummyTestDetailSerializer,
            "update": DummyTestDetailSerializer,
            "partial_update": DummyTestDetailSerializer,
        }

        config = create_schema_view_config(
            model_class=MockSerializerModel,
            display_config={
                "tag": "Test Models",
            },
            request_serializers=req_serializers,
            response_serializers=res_serializers,
            additional_responses=additional_responses,
        )

        expected_operations = [
            "list",
            "create",
            "retrieve",
            "update",
            "partial_update",
            "destroy",
        ]
        for operation in expected_operations:
            assert operation in config
            assert callable(config[operation])

    def test_config_request_response_serializers(self):
        req_serializers: RequestSerializersConfig = {
            "create": DummyTestWriteSerializer,
            "update": DummyTestWriteSerializer,
            "partial_update": DummyTestWriteSerializer,
        }

        res_serializers: ResponseSerializersConfig = {
            "create": DummyTestDetailSerializer,
            "list": DummyTestSerializer,
            "retrieve": DummyTestDetailSerializer,
            "update": DummyTestDetailSerializer,
            "partial_update": DummyTestDetailSerializer,
        }

        config = create_schema_view_config(
            model_class=MockSerializerModel,
            display_config={
                "tag": "Test Models",
            },
            request_serializers=req_serializers,
            response_serializers=res_serializers,
        )

        crud_operations = [
            "list",
            "create",
            "retrieve",
            "update",
            "partial_update",
            "destroy",
        ]
        for operation in crud_operations:
            assert operation in config
            assert callable(config[operation])

    def test_config_function_returns_decorator(self):
        req_serializers: RequestSerializersConfig = {
            "create": DummyTestWriteSerializer,
            "update": DummyTestWriteSerializer,
            "partial_update": DummyTestWriteSerializer,
        }

        res_serializers: ResponseSerializersConfig = {
            "create": DummyTestDetailSerializer,
            "list": DummyTestSerializer,
            "retrieve": DummyTestDetailSerializer,
            "update": DummyTestDetailSerializer,
            "partial_update": DummyTestDetailSerializer,
        }

        config = create_schema_view_config(
            model_class=MockSerializerModel,
            display_config={
                "tag": "Test Models",
            },
            request_serializers=req_serializers,
            response_serializers=res_serializers,
        )

        @config["list"]
        def dummy_list_view():
            pass

        @config["create"]
        def dummy_create_view():
            pass

        assert callable(dummy_list_view)
        assert callable(dummy_create_view)



# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('country', '0006_remove_country_country_cou_alpha_2_398383_idx_and_more'),
        ('order', '0013_alter_order_floor_alter_order_location_type'),
        ('pay_way', '0007_rename_payway_created_at_idx_payway_created_at_ix_and_more'),
        ('product', '0022_alter_productfavourite_options_and_more'),
        ('region', '0006_rename_region_created_at_idx_region_created_at_ix_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='order',
            name='order_search_gin',
        ),
        migrations.RemoveIndex(
            model_name='orderhistory',
            name='order_order_order_i_4345b0_idx',
        ),
        migrations.RemoveIndex(
            model_name='orderhistory',
            name='order_order_order_i_5a04bb_idx',
        ),
        migrations.RemoveIndex(
            model_name='orderitem',
            name='order_order_product_ed1645_idx',
        ),
        migrations.RemoveIndex(
            model_name='orderitem',
            name='order_order_is_refu_38a302_idx',
        ),
        migrations.RemoveIndex(
            model_name='orderitemhistory',
            name='order_order_order_i_b122c2_idx',
        ),
        migrations.RenameIndex(
            model_name='order',
            new_name='order_created_at_ix',
            old_name='order_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='order',
            new_name='order_updated_at_ix',
            old_name='order_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='order',
            new_name='order_status_ix',
            old_name='order_order_status_3ebebc_btree',
        ),
        migrations.RenameIndex(
            model_name='order',
            new_name='order_doc_type_ix',
            old_name='order_order_documen_0efd90_btree',
        ),
        migrations.RenameIndex(
            model_name='order',
            new_name='order_status_upd_ix',
            old_name='order_order_status__c9f5b7_btree',
        ),
        migrations.RenameIndex(
            model_name='order',
            new_name='order_payment_status_ix',
            old_name='order_order_payment_c12f17_btree',
        ),
        migrations.RenameIndex(
            model_name='orderhistory',
            new_name='orderhistory_created_at_ix',
            old_name='orderhistory_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='orderhistory',
            new_name='orderhistory_updated_at_ix',
            old_name='orderhistory_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='orderitem',
            new_name='orderitem_created_at_ix',
            old_name='orderitem_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='orderitem',
            new_name='orderitem_updated_at_ix',
            old_name='orderitem_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='orderitem',
            new_name='orderitem_sort_order_ix',
            old_name='orderitem_sort_order_idx',
        ),
        migrations.RenameIndex(
            model_name='orderitemhistory',
            new_name='orderitemhistory_created_at_ix',
            old_name='orderitemhistory_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='orderitemhistory',
            new_name='orderitemhistory_updated_at_ix',
            old_name='orderitemhistory_updated_at_idx',
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['user'], name='order_user_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['pay_way'], name='order_pay_way_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['country'], name='order_country_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['region'], name='order_region_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['user', 'status'], name='order_user_status_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['status', 'payment_status'], name='order_status_payment_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['tracking_number'], name='order_tracking_num_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['payment_id'], name='order_payment_id_ix'),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.GinIndex(fields=['first_name', 'last_name'], name='order_name_search_ix', opclasses=['gin_trgm_ops', 'gin_trgm_ops']),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.GinIndex(fields=['email'], name='order_email_search_ix', opclasses=['gin_trgm_ops']),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.GinIndex(fields=['street'], name='order_address_street_ix', opclasses=['gin_trgm_ops']),
        ),
        migrations.AddIndex(
            model_name='order',
            index=django.contrib.postgres.indexes.GinIndex(fields=['city', 'place'], name='order_address_location_ix', opclasses=['gin_trgm_ops', 'gin_trgm_ops']),
        ),
        migrations.AddIndex(
            model_name='orderhistory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['order', 'change_type'], name='ord_hist_ord_chtype_ix'),
        ),
        migrations.AddIndex(
            model_name='orderhistory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['order', '-created_at'], name='ord_hist_ord_crtd_ix'),
        ),
        migrations.AddIndex(
            model_name='orderhistory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['change_type'], name='ord_hist_chtype_ix'),
        ),
        migrations.AddIndex(
            model_name='orderitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['product'], name='order_item_product_ix'),
        ),
        migrations.AddIndex(
            model_name='orderitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['is_refunded'], name='order_item_is_refunded_ix'),
        ),
        migrations.AddIndex(
            model_name='orderitemhistory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['order_item', '-created_at'], name='ord_item_hist_item_created_ix'),
        ),
        migrations.AddIndex(
            model_name='orderitemhistory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['change_type'], name='ord_item_hist_change_type_ix'),
        ),
    ]



# Generated by Django 5.2.1 on 2025-06-02 19:38

import django.contrib.postgres.indexes
import django.db.models.deletion
import parler.fields
import parler.models
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0014_alter_useraddress_options_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubscriptionTopic',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('slug', models.SlugField(help_text="Unique identifier for the topic (e.g., 'weekly-newsletter')", unique=True, verbose_name='Slug')),
                ('category', models.CharField(choices=[('marketing', 'Marketing Campaigns'), ('product', 'Product Updates'), ('account', 'Account Updates'), ('system', 'System Notifications'), ('newsletter', 'Newsletter'), ('promotional', 'Promotional'), ('other', 'Other')], default='other', help_text='Category of the subscription topic', max_length=20, verbose_name='Category')),
                ('is_active', models.BooleanField(default=True, help_text='Whether this topic is currently available for subscription', verbose_name='Active')),
                ('is_default', models.BooleanField(default=False, help_text='Whether new users are automatically subscribed to this topic', verbose_name='Default Subscription')),
                ('requires_confirmation', models.BooleanField(default=False, help_text='Whether subscription to this topic requires email confirmation', verbose_name='Requires Confirmation')),
            ],
            options={
                'verbose_name': 'Subscription Topic',
                'verbose_name_plural': 'Subscription Topics',
                'ordering': ['-created_at'],
                'indexes': [django.contrib.postgres.indexes.BTreeIndex(fields=['created_at'], name='subscriptiontopic_created_at_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['updated_at'], name='subscriptiontopic_updated_at_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['slug'], name='sub_topic_slug_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['is_active'], name='sub_topic_active_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['category'], name='sub_topic_category_ix')],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SubscriptionTopicTranslation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(help_text='Human-readable name for the topic', max_length=100, verbose_name='Name')),
                ('description', models.TextField(blank=True, help_text='Detailed description of what this subscription includes', verbose_name='Description')),
                ('master', parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='user.subscriptiontopic')),
            ],
            options={
                'verbose_name': 'Subscription Topic Translation',
                'db_table': 'user_subscriptiontopic_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
                'unique_together': {('language_code', 'master')},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='UserSubscription',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('pending', 'Pending Confirmation'), ('unsubscribed', 'Unsubscribed'), ('bounced', 'Bounced')], default='active', max_length=20, verbose_name='Status')),
                ('subscribed_at', models.DateTimeField(auto_now_add=True, verbose_name='Subscribed At')),
                ('unsubscribed_at', models.DateTimeField(blank=True, null=True, verbose_name='Unsubscribed At')),
                ('confirmation_token', models.CharField(blank=True, help_text='Token for email confirmation if required', max_length=64, verbose_name='Confirmation Token')),
                ('metadata', models.JSONField(blank=True, default=dict, help_text='Additional subscription preferences or data', verbose_name='Metadata')),
                ('topic', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscribers', to='user.subscriptiontopic', verbose_name='Topic')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'User Subscription',
                'verbose_name_plural': 'User Subscriptions',
                'ordering': ['-created_at'],
                'indexes': [django.contrib.postgres.indexes.BTreeIndex(fields=['created_at'], name='usersubscription_created_at_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['updated_at'], name='usersubscription_updated_at_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['user', 'status'], name='user_sub_user_status_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['topic', 'status'], name='user_sub_topic_status_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['status'], name='user_sub_status_ix'), django.contrib.postgres.indexes.BTreeIndex(fields=['confirmation_token'], name='user_sub_token_ix')],
                'unique_together': {('user', 'topic')},
            },
        ),
    ]



import math

from rest_framework import pagination
from rest_framework.response import Response


class LimitOffsetPaginator(pagination.LimitOffsetPagination):
    max_limit = 100

    def get_paginated_response(self, data):
        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.count,
                "total_pages": math.ceil(self.count / self.limit),
                "page_size": self.limit,
                "page_total_results": len(data),
                "page": math.ceil(self.offset / self.limit) + 1,
                "results": data,
            }
        )

    def get_paginated_response_schema(self, schema):
        return {
            "type": "object",
            "required": ["count", "results"],
            "properties": {
                "links": {
                    "type": "object",
                    "properties": {
                        "next": {
                            "type": "string",
                            "nullable": True,
                            "format": "uri",
                            "example": "http://api.example.org/accounts/?{limit_query_param}=2&{offset_query_param}=4".format(
                                limit_query_param=self.limit_query_param,
                                offset_query_param=self.offset_query_param,
                            ),
                        },
                        "previous": {
                            "type": "string",
                            "nullable": True,
                            "format": "uri",
                            "example": "http://api.example.org/accounts/?{limit_query_param}=2&{offset_query_param}=0".format(
                                limit_query_param=self.limit_query_param,
                                offset_query_param=self.offset_query_param,
                            ),
                        },
                    },
                },
                "count": {
                    "type": "integer",
                    "example": 123,
                },
                "total_pages": {
                    "type": "integer",
                    "example": 123,
                },
                "page_size": {
                    "type": "integer",
                    "example": 123,
                },
                "page_total_results": {
                    "type": "integer",
                    "example": 123,
                },
                "page": {
                    "type": "integer",
                    "example": 123,
                },
                "results": schema,
            },
        }



from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters


class TimeStampFilterMixin:
    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text=_("Filter items created after this date"),
    )
    created_before = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text=_("Filter items created before this date"),
    )
    updated_after = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="gte",
        help_text=_("Filter items updated after this date"),
    )
    updated_before = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="lte",
        help_text=_("Filter items updated before this date"),
    )


class UUIDFilterMixin:
    uuid = filters.UUIDFilter(
        field_name="uuid",
        help_text=_("Filter by exact UUID"),
    )


class SortableFilterMixin:
    sort_order = filters.NumberFilter(
        field_name="sort_order",
        help_text=_("Filter by exact sort order"),
    )
    sort_order_min = filters.NumberFilter(
        field_name="sort_order",
        lookup_expr="gte",
        help_text=_("Filter items with sort order greater than or equal to"),
    )
    sort_order_max = filters.NumberFilter(
        field_name="sort_order",
        lookup_expr="lte",
        help_text=_("Filter items with sort order less than or equal to"),
    )


class PublishableFilterMixin:
    is_published = filters.BooleanFilter(
        field_name="is_published",
        help_text=_("Filter by published status"),
    )
    published_after = filters.DateTimeFilter(
        field_name="published_at",
        lookup_expr="gte",
        help_text=_("Filter items published after this date"),
    )
    published_before = filters.DateTimeFilter(
        field_name="published_at",
        lookup_expr="lte",
        help_text=_("Filter items published before this date"),
    )
    currently_published = filters.BooleanFilter(
        method="filter_currently_published",
        help_text=_(
            "Filter items that are currently published (published_at <= now and is_published=True)"
        ),
    )

    def filter_currently_published(self, queryset, name, value):
        if value:
            return queryset.published()
        return queryset


class SoftDeleteFilterMixin:
    is_deleted = filters.BooleanFilter(
        field_name="is_deleted",
        help_text=_("Filter by deleted status"),
    )
    include_deleted = filters.BooleanFilter(
        method="filter_include_deleted",
        help_text=_("Include deleted items in results"),
    )
    deleted_after = filters.DateTimeFilter(
        field_name="deleted_at",
        lookup_expr="gte",
        help_text=_("Filter items deleted after this date"),
    )
    deleted_before = filters.DateTimeFilter(
        field_name="deleted_at",
        lookup_expr="lte",
        help_text=_("Filter items deleted before this date"),
    )

    def filter_include_deleted(self, queryset, name, value):
        if value:
            return queryset.model.objects.all_with_deleted()
        return queryset


class MetaDataFilterMixin:
    metadata_has_key = filters.CharFilter(
        field_name="metadata",
        lookup_expr="has_key",
        help_text=_("Filter items where metadata contains the specified key"),
    )
    metadata_has_keys = filters.CharFilter(
        field_name="metadata",
        lookup_expr="has_keys",
        help_text=_(
            "Filter items where metadata contains all specified keys (comma-separated)"
        ),
        method="filter_metadata_has_keys",
    )
    metadata_has_any_keys = filters.CharFilter(
        field_name="metadata",
        lookup_expr="has_any_keys",
        help_text=_(
            "Filter items where metadata contains any of the specified keys (comma-separated)"
        ),
        method="filter_metadata_has_any_keys",
    )
    metadata_contains = filters.CharFilter(
        method="filter_metadata_contains",
        help_text=_(
            "Filter items where metadata contains the specified JSON (as string)"
        ),
    )
    private_metadata_has_key = filters.CharFilter(
        field_name="private_metadata",
        lookup_expr="has_key",
        help_text=_(
            "Filter items where private metadata contains the specified key"
        ),
    )

    def filter_metadata_has_keys(self, queryset, name, value):
        if value:
            keys = [k.strip() for k in value.split(",")]
            return queryset.filter(metadata__has_keys=keys)
        return queryset

    def filter_metadata_has_any_keys(self, queryset, name, value):
        if value:
            keys = [k.strip() for k in value.split(",")]
            return queryset.filter(metadata__has_any_keys=keys)
        return queryset

    def filter_metadata_contains(self, queryset, name, value):
        if value:
            import json

            try:
                json_data = json.loads(value)
                return queryset.filter(metadata__contains=json_data)
            except json.JSONDecodeError:
                return queryset.none()
        return queryset


class BaseTimeStampFilterSet(TimeStampFilterMixin, filters.FilterSet):
    class Meta:
        abstract = True
        fields = {
            "created_at": ["gte", "lte", "date", "year", "month", "day"],
            "updated_at": ["gte", "lte", "date", "year", "month", "day"],
        }


class BasePublishableTimeStampFilterSet(
    PublishableFilterMixin, TimeStampFilterMixin, filters.FilterSet
):
    class Meta:
        abstract = True
        fields = {
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "published_at": ["gte", "lte", "date"],
            "is_published": ["exact"],
        }


class BaseSoftDeleteTimeStampFilterSet(
    SoftDeleteFilterMixin, TimeStampFilterMixin, filters.FilterSet
):
    class Meta:
        abstract = True
        fields = {
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "deleted_at": ["gte", "lte", "date"],
            "is_deleted": ["exact"],
        }


class BaseFullFilterSet(
    TimeStampFilterMixin,
    UUIDFilterMixin,
    SortableFilterMixin,
    PublishableFilterMixin,
    SoftDeleteFilterMixin,
    MetaDataFilterMixin,
    filters.FilterSet,
):
    class Meta:
        abstract = True
        fields = {
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "published_at": ["gte", "lte", "date"],
            "deleted_at": ["gte", "lte", "date"],
            "is_published": ["exact"],
            "is_deleted": ["exact"],
            "sort_order": ["exact", "gte", "lte"],
            "uuid": ["exact"],
        }



import random
import string


def generate_random_password(
    length=12, use_digits=False, use_special_chars=False
):
    chars = string.ascii_letters

    if use_digits:
        chars += string.digits

    if use_special_chars:
        chars += string.punctuation

    if use_digits:
        password = random.choice(string.digits) + "".join(
            random.choice(chars) for _ in range(length - 1)
        )
    else:
        password = "".join(random.choice(chars) for _ in range(length))

    password_list = list(password)
    random.shuffle(password_list)
    password = "".join(password_list)

    return password
